@@ -70,24 +70,23 @@
 int Slrn_Query_Group_Cutoff = 100;
 int Slrn_Groups_Dirty;	       /* greater than 0 if need to write newsrc */
 int Slrn_List_Active_File = 0;
-int Slrn_Use_Xgtitle = 0;
 int Slrn_Write_Newsrc_Flags = 0;       /* if 1, do not save unsubscribed 
 					* if 2, do not save new unsubscribed.
 					*/
 
 int Slrn_Display_Cursor_Bar;
-int Slrn_Group_Description_Column = 40;/* column where group descr start */
 char *Slrn_Group_Help_Line;
+char *Slrn_Group_Status_Line;
 
 Slrn_Group_Type *Slrn_Group_Current_Group;
 static SLscroll_Window_Type Group_Window;
 static Slrn_Group_Type *Groups;
 
-int Slrn_Group_Display_Descriptions = 1;
 int Slrn_No_Backups = 0;
 
 int Slrn_Unsubscribe_New_Groups = 0;
 int Slrn_Check_New_Groups = 1;
+int Slrn_Drop_Bogus_Groups = 1;
 
 SLKeyMap_List_Type *Slrn_Group_Keymap;
 int *Slrn_Prefix_Arg_Ptr;
@@ -99,7 +98,8 @@ int *Slrn_Prefix_Arg_Ptr;
 static Slrn_Group_Type *Group_Hash_Table [GROUP_HASH_TABLE_SIZE];
 
 static unsigned int Last_Cursor_Row;
-static int Groups_Hidden;	       /* if true, hide groups with no arts */
+static int Groups_Hidden;	/* if 1, hide groups with no articles;
+				 * if 2, groups are hidden by user request */
 static int Kill_After_Max = 1;
 
 typedef struct Unsubscribed_Slrn_Group_Type
@@ -119,6 +119,8 @@ static void group_update_screen (void);
 static void group_quick_help (void);
 static void save_newsrc_cmd (void);
 static void read_and_parse_active (int);
+static int  parse_active_line (char *, unsigned int *, int *, int *);
+static void remove_group_entry (Slrn_Group_Type *);
 
 /*}}}*/
 
@@ -349,7 +351,7 @@ static int group_update_range (Slrn_Group_Type *g, int min, int max) /*{{{*/
      {
 	if (g->unread > 0)
 	  g->flags &= ~GROUP_HIDDEN;
-	else if (Groups_Hidden && (g != Slrn_Group_Current_Group))
+	else if ((Groups_Hidden & 1) && (g != Slrn_Group_Current_Group))
 	  g->flags |= GROUP_HIDDEN;
      }
    
@@ -377,10 +379,17 @@ static int group_sync_group_with_server (Slrn_Group_Type *g, int *minp, int *max
    if (status == -1)
      return -1;
    
-   if (status != OK_GROUP)
+   if (status == ERR_NOGROUP)
      {
-	g->flags |= GROUP_UNSUBSCRIBED;
-	slrn_error ("Group %s appears to be bogus.", group);
+	slrn_error ("Group %s is bogus%s.", group,
+		    Slrn_Drop_Bogus_Groups ? " - dropping it" : "");
+	if (Slrn_Drop_Bogus_Groups)
+	  remove_group_entry (g);
+	return -1;
+     }
+   else if (status != OK_GROUP)
+     {
+	slrn_error ("Could not enter group %s.", group);
 	return -1;
      }
    
@@ -451,6 +460,14 @@ void slrn_uncatchup_group (void) /*{{{*/
 /*}}}*/
 
 /*{{{ Misc Utility Functions */
+static void find_line_num (void) /*{{{*/
+{
+   Group_Window.lines = (SLscroll_Type *) Groups;
+   Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
+   (void) SLscroll_find_line_num (&Group_Window);
+}
+
+/*}}}*/
 
 static Slrn_Group_Type *find_group_entry (char *name, unsigned int len) /*{{{*/
 {
@@ -500,15 +517,16 @@ static Slrn_Group_Type *create_group_entry (char *name, unsigned int len, /*{{{*
 	int status;
 	
 	status = Slrn_Server_Obj->sv_select_group (g->group_name, &min, &max);
-	if (status != OK_GROUP)
+	if (status == ERR_NOGROUP)
 	  {
-	     if (status != -1)
+	     slrn_error ("Group %s is bogus%s.", g->group_name,
+			 Slrn_Drop_Bogus_Groups ? " - ignoring it" : "");
+	     if (Slrn_Drop_Bogus_Groups)
 	       {
 		  Slrn_Groups_Dirty = 1;
-		  slrn_error ("Group %s may be bogus.", g->group_name);
+		  free_newsgroup_type (g);
+		  return NULL;
 	       }
-	     free_newsgroup_type (g);
-	     return NULL;
 	  }
      }
    g->range.min = min;
@@ -533,16 +551,52 @@ static Slrn_Group_Type *create_group_entry (char *name, unsigned int len, /*{{{*
      }
    else
      {
-	if (Slrn_Group_Current_Group == NULL) Slrn_Group_Current_Group = Groups;
-	g->next = Slrn_Group_Current_Group->next;
-	if (g->next != NULL) g->next->prev = g;
-	Slrn_Group_Current_Group->next = g;
-	g->prev = Slrn_Group_Current_Group;
+	if (Slrn_Group_Current_Group == NULL) /* insert on top */
+	  {
+	     g->prev = NULL;
+	     g->next = Groups;
+	     Groups->prev = g;
+	     Groups = g;
+	  }
+	else /* insert after current group */
+	  {
+	     g->next = Slrn_Group_Current_Group->next;
+	     if (g->next != NULL) g->next->prev = g;
+	     Slrn_Group_Current_Group->next = g;
+	     g->prev = Slrn_Group_Current_Group;
+	  }
      }
    Slrn_Group_Current_Group = g;
    return g;
 }
 
+/*}}}*/
+static void remove_group_entry (Slrn_Group_Type *g) /*{{{*/
+{
+   Slrn_Group_Type *tmp;
+   if (g == Groups)
+     Groups = g->next;
+   if (g == Slrn_Group_Current_Group)
+     Slrn_Group_Current_Group = g->next != NULL ? g->next : g->prev;
+   if (g->prev != NULL)
+     g->prev->next = g->next;
+   if (g->next != NULL)
+     g->next->prev = g->prev;
+   if (g == (tmp = Group_Hash_Table[g->hash % GROUP_HASH_TABLE_SIZE]))
+     Group_Hash_Table[g->hash % GROUP_HASH_TABLE_SIZE] = g->hash_next;
+   else while (tmp != NULL)
+     {
+	if (g == tmp->hash_next)
+	  {
+	     tmp->hash_next = g->hash_next;
+	     break;
+	  }
+	tmp = tmp->hash_next;
+     }
+   free_newsgroup_type (g);
+   find_line_num ();
+   Slrn_Groups_Dirty = 1;
+}
 /*}}}*/
 static int add_group (char *name, unsigned int len, /*{{{*/
 		      unsigned int subscribe_flag, int create_flag)
@@ -553,10 +607,10 @@ static int add_group (char *name, unsigned int len, /*{{{*/
    g = find_group_entry (name, len);
    if (g == NULL)
      {
-	if (Slrn_List_Active_File)
+	if (Slrn_List_Active_File && Slrn_Drop_Bogus_Groups)
 	  {
 	     char *tmp_name;
-
+	     
 	     if (NULL != (tmp_name = slrn_strnmalloc (name, len, 1)))
 	       {
 		  slrn_error ("Group %s is bogus - ignoring it.", tmp_name);
@@ -663,19 +717,26 @@ static Slrn_Group_Type *place_group_in_newsrc_order (Slrn_Group_Type *last_group
 }
 
 /*}}}*/
-static Slrn_Group_Type *insert_new_groups (void) /*{{{*/
+static void insert_new_groups (void) /*{{{*/
 {
-   Slrn_Group_Type *last_group = NULL;
+   Slrn_Group_Type *last_group = Groups;
+   
+   while (last_group != NULL)
+     {
+	/* unmark new groups from previous run */
+	last_group->flags &= ~GROUP_NEW_GROUP_FLAG;
+	last_group = last_group->next;
+     }
    
    if (Unsubscribed_Groups != NULL)
      {
 	unsigned int subscribe_flag;
 	Unsubscribed_Slrn_Group_Type *ug = Unsubscribed_Groups, *ugnext;
-	     
+	
 	if (Slrn_Unsubscribe_New_Groups)
 	  subscribe_flag = GROUP_UNSUBSCRIBED | GROUP_NEW_GROUP_FLAG;
 	else subscribe_flag = GROUP_NEW_GROUP_FLAG;
-	     
+	
 	while (ug != NULL)
 	  {
 	     ugnext = ug->next;
@@ -688,20 +749,10 @@ static Slrn_Group_Type *insert_new_groups (void) /*{{{*/
 	  }
 	Unsubscribed_Groups = NULL;
      }
-   
-   return last_group;
 }
 
 /*}}}*/
 
-static void find_line_num (void) /*{{{*/
-{
-   Group_Window.lines = (SLscroll_Type *) Groups;
-   Group_Window.current_line = (SLscroll_Type *) Slrn_Group_Current_Group;
-   (void) SLscroll_find_line_num (&Group_Window);
-}
-
-/*}}}*/
 static void init_group_win_struct (void) /*{{{*/
 {
    Group_Window.nrows = SLtt_Screen_Rows - 3;
@@ -746,53 +797,6 @@ static SLRegexp_Type *read_group_regexp (char *prompt, char *origpat) /*{{{*/
    return slrn_compile_regexp_pattern (slrn_fix_regexp (pattern));
 }
 
-/*}}}*/
-static Slrn_Group_Type *process_xgtitle_info (void) /*{{{*/
-{
-   char buf [NNTP_BUFFER_SIZE];
-   Slrn_Group_Type *first = NULL, *save = Slrn_Group_Current_Group;
-   
-   while (1)
-     {
-	char *b, ch;
-	unsigned int len;
-	Slrn_Group_Type *g;
-	int status;
-
-	status = Slrn_Server_Obj->sv_read_line(buf, sizeof(buf));
-	if (status <= 0)
-	  break;
-
-	b = buf;
-	while (((ch = *b) != 0)
-	       && (ch != ' ') 
-	       && (ch != '\n') 
-	       && (ch != '\t'))
-	  b++;
-	
-	len = (unsigned int) (b - buf);
-	if (len == 0) continue;
-	*b = 0;
-	
-	g = create_group_entry (buf, len,
-				-1, -1, 0, 0);
-
-	if (g != NULL)
-	  {
-	     g->flags &= ~GROUP_HIDDEN;
-	     if ((first == NULL) && (g->flags & GROUP_UNSUBSCRIBED))
-	       first = g;
-	  }
-     }
-   
-   if (save != Slrn_Group_Current_Group)
-     {
-	Slrn_Group_Current_Group = save;
-	find_line_num ();
-     }
-   return first;
-}
-
 /*}}}*/
 
 static void add_unsubscribed_group (char *name) /*{{{*/
@@ -818,6 +822,20 @@ static void add_unsubscribed_group (char *name) /*{{{*/
 
 /*}}}*/
 
+static char *Group_Display_Formats [SLRN_MAX_DISPLAY_FORMATS];
+static unsigned int Group_Format_Number;
+
+int slrn_set_group_format (unsigned int num, char *fmt)
+{
+   return slrn_set_display_format (Group_Display_Formats, num, fmt);
+}
+
+static void toggle_group_formats (void)
+{
+   Group_Format_Number = slrn_toggle_format (Group_Display_Formats,
+					     Group_Format_Number);
+}
+
 int slrn_group_search (char *str) /*{{{*/
 {
    SLsearch_Type st;
@@ -926,7 +944,7 @@ int slrn_group_select_group (void) /*{{{*/
 	     return 0;
 	  }
 	
-	if ((0 == (prefix & 1))
+	if ((0 == prefix)
 	    && (Slrn_Group_Current_Group->unread != 0))
 	  {
 	     r = Slrn_Group_Current_Group->range.next;
@@ -951,8 +969,7 @@ int slrn_group_select_group (void) /*{{{*/
    if (ret == -2) 
      slrn_catchup_group ();
    
-   if (ret == 0) return 0;
-   return -1;
+   return ret;
 }
 
 /*}}}*/
@@ -968,6 +985,12 @@ void slrn_select_next_group (void) /*{{{*/
 	
 	if (0 == slrn_group_select_group ())
 	  break;
+	else if (SLang_Error == INTRINSIC_ERROR)
+	  /* all articles killed by scorefile, so proceed */
+	  {
+	     SLang_Error = 0;
+	     slrn_clear_message ();
+	  }
      }
 }
 
@@ -990,7 +1013,6 @@ void slrn_select_prev_group (void) /*{{{*/
 
 /*}}}*/
 
-
 /*{{{ Interactive commands */
 
 void slrn_group_quit (void) /*{{{*/
@@ -1222,6 +1244,10 @@ static void move_group_cmd (void) /*{{{*/
    if (Slrn_Batch) return;
    if (Slrn_Group_Current_Group == NULL) return;
    from = Slrn_Group_Current_Group;
+
+   /* Already centering the window here should avoid confusing the user */
+   Group_Window.top_window_line = NULL;
+   slrn_update_screen ();
    
    while (1)
      {
@@ -1294,14 +1320,15 @@ static void subscribe (void) /*{{{*/
    SLRegexp_Type *re;
    Slrn_Group_Type *g;
    
-   if (Slrn_Group_Current_Group == NULL) return;
-   
    if (Slrn_Prefix_Arg_Ptr == NULL)
      {
-	Slrn_Group_Current_Group->flags &= ~GROUP_UNSUBSCRIBED;
-	Slrn_Group_Current_Group->flags |= GROUP_TOUCHED;
-	slrn_group_down_n (1);
-	Slrn_Groups_Dirty = 1;
+	if (Slrn_Group_Current_Group != NULL)
+	  {
+	     Slrn_Group_Current_Group->flags &= ~GROUP_UNSUBSCRIBED;
+	     Slrn_Group_Current_Group->flags |= GROUP_TOUCHED;
+	     slrn_group_down_n (1);
+	     Slrn_Groups_Dirty = 1;
+	  }
 	return;
      }
    
@@ -1320,6 +1347,7 @@ static void subscribe (void) /*{{{*/
 		  g->flags &= ~GROUP_HIDDEN;
 		  g->flags &= ~GROUP_UNSUBSCRIBED;
 		  g->flags |= GROUP_TOUCHED;
+		  Slrn_Groups_Dirty = 1;
 	       }
 	  }
 	g = g->next;
@@ -1431,6 +1459,14 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 	     g = g->next;
 	  }
      }
+   else if (hide_flag != -1)
+     {
+	while (g != NULL)
+	  {
+	     if (g->flags & GROUP_UNSUBSCRIBED) g->flags &= ~GROUP_HIDDEN;
+	     g = g->next;
+	  }
+     }
    else
      {
 	SLRegexp_Type *re;
@@ -1444,20 +1480,37 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 	  }
 	
 	if ((Slrn_List_Active_File == 0)
-	    && Slrn_Use_Xgtitle
-	    && (OK_XGTITLE == Slrn_Server_Obj->sv_xgtitle_cmd (origpat)))
+	    && (OK_GROUPS == Slrn_Server_Obj->sv_list_active (origpat)))
 	  {
-	     first_found = process_xgtitle_info ();
+	     char buf [NNTP_BUFFER_SIZE];
+	     Slrn_Group_Type *save = Slrn_Group_Current_Group;
+	     Slrn_Group_Current_Group = NULL;
+	     
+	     while (Slrn_Server_Obj->sv_read_line (buf, sizeof (buf)) > 0)
+	       {
+		  unsigned int len;
+		  int min, max;
+
+		  parse_active_line (buf, &len, &min, &max);
+		  g = create_group_entry (buf, len, min, max, 0, 0);
+		  
+		  if (g != NULL)
+		    {
+		       g->flags &= ~GROUP_HIDDEN;
+		       if ((first_found == NULL) && (g->flags & GROUP_UNSUBSCRIBED))
+			 first_found = g;
+		    }
+	       }
+	     
+	     Slrn_Group_Current_Group = save;
 	  }
 	else while (g != NULL)
 	  {
-	     if (g->flags & GROUP_UNSUBSCRIBED)
+	     if (NULL != slrn_regexp_match (re, g->group_name))
 	       {
-		  if (NULL != slrn_regexp_match (re, g->group_name))
-		    {
-		       if (first_found == NULL) first_found = g;
-		       g->flags &= ~GROUP_HIDDEN;
-		    }
+		  if ((first_found == NULL) && (g->flags & GROUP_UNSUBSCRIBED))
+		    first_found = g;
+		  g->flags &= ~GROUP_HIDDEN;
 	       }
 	     g = g->next;
 	  }
@@ -1496,22 +1549,27 @@ static void toggle_list_all_groups1 (int hide_flag) /*{{{*/
 
 static void toggle_list_all_groups (void) /*{{{*/
 {
-   int mode;
+   int mode = -1;
+   
    if (Slrn_Prefix_Arg_Ptr != NULL)
-     mode = 1;
-   else mode = -1;
+     {
+	mode = *Slrn_Prefix_Arg_Ptr;
+	Slrn_Prefix_Arg_Ptr = NULL;
+	if (mode == 2) mode = 0;
+     }
    
    toggle_list_all_groups1 (mode);
 }
 
 /*}}}*/
 
-static void toggle_group_display (void) /*{{{*/
+void slrn_hide_current_group (void) /*{{{*/
 {
-   Slrn_Group_Display_Descriptions = !Slrn_Group_Display_Descriptions;
-   Slrn_Full_Screen_Update = 1;
+   if (Slrn_Group_Current_Group == NULL) return;
+   Groups_Hidden |= 2;
+   Slrn_Group_Current_Group->flags |= GROUP_HIDDEN;
+   set_current_group ();
 }
-
 /*}}}*/
 
 static void toggle_hide_groups (void) /*{{{*/
@@ -1537,8 +1595,7 @@ static void toggle_hide_groups (void) /*{{{*/
      {
 	while (g != NULL)
 	  {
-	     if ((g->unread == 0)
-		 && ((g->flags & GROUP_UNSUBSCRIBED) == 0))
+	     if ((g->flags & GROUP_UNSUBSCRIBED) == 0)
 	       g->flags &= ~GROUP_HIDDEN;
 	     
 	     g = g->next;
@@ -1550,7 +1607,6 @@ static void toggle_hide_groups (void) /*{{{*/
 
 /*}}}*/
 
-
 static void select_group_cmd (void)
 {
    if (-1 == slrn_group_select_group ())
@@ -1559,8 +1615,9 @@ static void select_group_cmd (void)
 
 void slrn_post_cmd (void) /*{{{*/
 {
-   char *name, *dist = NULL;	       /* "world"; */
+   char *name;
    char group[256];
+   char followupto[256];
    char subj[256];
    
    if (Slrn_Post_Obj->po_can_post == 0)
@@ -1587,9 +1644,17 @@ void slrn_post_cmd (void) /*{{{*/
      name = "";
    slrn_strncpy (group, name, sizeof (group));
    if (slrn_read_input ("Newsgroup", NULL, group, 1, 1) <= 0) return;
+   if (strchr (group, ',') != NULL)
+     {
+	slrn_strncpy (followupto, name, sizeof (followupto));
+	(void) slrn_read_input ("Followup-To", NULL, followupto, 1, 1);
+     }
+   else
+     *followupto = '\0';
+   
    *subj = 0; if (slrn_read_input ("Subject", NULL, subj, 1, 0) <= 0) return;
    
-   (void) slrn_post (group, subj, dist);
+   (void) slrn_post (group, followupto, subj);
 }
 
 /*}}}*/
@@ -1691,6 +1756,7 @@ static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
    A_KEY("catchup", catch_up),
    A_KEY("digit_arg", slrn_digit_arg),
    A_KEY("eob", group_eob),
+   A_KEY("evaluate_cmd", slrn_evaluate_cmd),
    A_KEY("group_search", group_search_cmd),
    A_KEY("group_search_forward", group_search_cmd),
    A_KEY("help", slrn_group_help),
@@ -1709,7 +1775,7 @@ static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
    A_KEY("select_group", select_group_cmd),
    A_KEY("subscribe", subscribe),
    A_KEY("suspend", slrn_suspend_cmd),
-   A_KEY("toggle_group_display", toggle_group_display),
+   A_KEY("toggle_group_formats", toggle_group_formats),
    A_KEY("toggle_hidden", toggle_hide_groups),
    A_KEY("toggle_list_all", toggle_list_all_groups),
    A_KEY("toggle_scoring", toggle_scoring),
@@ -1722,6 +1788,7 @@ static SLKeymap_Function_Type Group_Functions [] = /*{{{*/
    A_KEY("group_eob", group_eob),
    A_KEY("pagedown", group_pagedown),
    A_KEY("pageup", group_pageup),
+   A_KEY("toggle_group_display", toggle_group_formats),
    A_KEY("uncatch_up", uncatch_up),
    A_KEY("up", group_up),
 #endif
@@ -1785,7 +1852,7 @@ static void group_mouse_left (void)
 
 static void group_mouse_middle (void)
 {
-   group_mouse (toggle_group_display, toggle_hide_groups, select_group_cmd);
+   group_mouse (toggle_group_formats, toggle_hide_groups, select_group_cmd);
 #if 1
    /* Make up for buggy rxvt which have problems with the middle key. */
    if (NULL != getenv ("COLORTERM"))
@@ -1847,7 +1914,7 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("^K\033OA", (FVOID_STAR) group_bob, Slrn_Group_Keymap);
    SLkm_define_key  ("^K\033[B", (FVOID_STAR) group_eob, Slrn_Group_Keymap);
    SLkm_define_key  ("^K\033OB", (FVOID_STAR) group_eob, Slrn_Group_Keymap);
-   SLkm_define_key  ("\033A", (FVOID_STAR) toggle_group_display, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033a", (FVOID_STAR) toggle_group_formats, Slrn_Group_Keymap);
    SLkm_define_key  ("\033>", (FVOID_STAR) group_eob, Slrn_Group_Keymap);
    SLkm_define_key  ("\033<", (FVOID_STAR) group_bob, Slrn_Group_Keymap);
    SLkm_define_key  ("^D", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
@@ -1859,7 +1926,9 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("\xE0I", (FVOID_STAR) group_pageup, Slrn_Group_Keymap);   
 #else
    SLkm_define_key  ("\033[6~", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033[G", (FVOID_STAR) group_pagedown, Slrn_Group_Keymap);
    SLkm_define_key  ("\033[5~", (FVOID_STAR) group_pageup, Slrn_Group_Keymap);
+   SLkm_define_key  ("\033[I", (FVOID_STAR) group_pageup, Slrn_Group_Keymap);
 #endif
    SLkm_define_key  ("m", (FVOID_STAR) move_group_cmd, Slrn_Group_Keymap);
    SLkm_define_key  ("^U", (FVOID_STAR) group_pageup, Slrn_Group_Keymap);
@@ -1882,6 +1951,7 @@ void slrn_init_group_mode (void) /*{{{*/
    SLkm_define_key  ("q", (FVOID_STAR) slrn_group_quit, Slrn_Group_Keymap);
    SLkm_define_key  ("^X^C", (FVOID_STAR) slrn_group_quit, Slrn_Group_Keymap);
    SLkm_define_key  ("^X^T", (FVOID_STAR) transpose_groups, Slrn_Group_Keymap);
+   SLkm_define_key  ("^X^[", (FVOID_STAR) slrn_evaluate_cmd, Slrn_Group_Keymap);
    SLkm_define_key  ("^R", (FVOID_STAR) slrn_redraw, Slrn_Group_Keymap);
    SLkm_define_key  ("^L", (FVOID_STAR) slrn_redraw, Slrn_Group_Keymap);
    SLkm_define_key  ("^P", (FVOID_STAR) group_up, Slrn_Group_Keymap);
@@ -1974,13 +2044,13 @@ void slrn_get_group_descriptions (void) /*{{{*/
 #endif
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "w", line, sizeof (line), 0)))
+   if (NULL == (fp = fopen (file, "w")))
      {
 	slrn_exit_error ("\
-Unable to create newsgroup description file:\n%s\n", line);
+Unable to create newsgroup description file:\n%s\n", file);
      }
    
-   fprintf (stdout, "\nCreating description file %s.\n", line);
+   fprintf (stdout, "\nCreating description file %s.\n", file);
    fprintf (stdout, "Getting newsgroup descriptions from server.\n\
 Note: This step may take some time if you have a slow connection!!!\n");
    
@@ -2061,24 +2131,23 @@ int slrn_read_group_descriptions (void) /*{{{*/
 #endif
    
    
-   if (NULL == (fp = slrn_open_home_file (file, "r", line, sizeof (line), 0)))
+   if (NULL == (fp = fopen (file, "r")))
      {
 	if (Slrn_Lib_Dir != NULL)
 	  {
 #ifdef VMS
 	     slrn_snprintf (file, sizeof (file), "%snewsgroups.dsc",
 			    Slrn_Lib_Dir);
-	     if (NULL == (fp = slrn_open_home_file (file, "r", line,
-						    sizeof (line), 0)))
+	     if (NULL == (fp = fopen (file, "r")))
 	       {
 		  slrn_snprintf (file, sizeof (file), "%snewsgroups-dsc",
 				 Slrn_Lib_Dir);
-		  fp = slrn_open_home_file (file, "r", line, sizeof (line), 0);
+		  fp = fopen (file, "r");
 	       }
 #else
 	     slrn_snprintf (file, sizeof (file), "%s/newsgroups.dsc",
 			    Slrn_Lib_Dir);
-	     fp = slrn_open_home_file (file, "r", line, sizeof (line), 0);
+	     fp = fopen (file, "r");
 #endif
 	  }
 	if (fp == NULL) return -1;
@@ -2132,7 +2201,7 @@ int slrn_get_new_groups (int create_flag) /*{{{*/
    
    
    if ((create_flag == 0)
-       && (NULL != (fp = slrn_open_home_file (file, "r", NULL, 0, 0))))
+       && (NULL != (fp = fopen (file, "r"))))
      {
 	char ch;
 	int i;
@@ -2253,8 +2322,11 @@ I expected to see see: NEWGROUPS yymmdd hhmmss GMT\n\
 I will patch the file up for you.\n", file);
      }
    
-   
-   if (NULL == (fp = slrn_open_home_file (file, "w", NULL, 0, 1)))
+#ifdef VMS
+   if (NULL == (fp = fopen (file, "w", "fop=cif")))
+#else
+   if (NULL == (fp = fopen (file, "w")))
+#endif
      {
 	slrn_exit_error ("Unable to open %s to record date.", file);
      }
@@ -2304,17 +2376,30 @@ static int parse_active_line (char *name, unsigned int *lenp, /*{{{*/
 
 /*}}}*/
 
+static char *Subscriptions [] =
+{
+   "news.answers",
+     "news.announce.newusers",
+     "news.newusers.questions",
+     "news.groups.questions",
+     "news.software.readers",
+     "alt.test",
+     NULL
+};
+
 static void read_and_parse_active (int create_flag) /*{{{*/
 {
    char line[NNTP_BUFFER_SIZE];
-   int count = 0;
+   int count = 0, ret;
    int initial_run = (Groups == NULL);
 
-   if (OK_GROUPS != Slrn_Server_Obj->sv_list_active ())
-     slrn_exit_error ("Server failed LIST ACTIVE.");
+   if (OK_GROUPS != (ret = Slrn_Server_Obj->sv_list_active (NULL)))
+     {
+	if (ret == ERR_NOAUTH)
+	  slrn_exit_error ("Server failed LIST ACTIVE - authorization missing.");
+	slrn_exit_error ("Server failed LIST ACTIVE.");
+     }
    
-   Kill_After_Max = 0;
-
    while (1)
      {
 	unsigned int len;
@@ -2354,25 +2439,46 @@ static void read_and_parse_active (int create_flag) /*{{{*/
 		  putc ('.', stdout);
 		  fflush (stdout);
 	       }
-	     
-	     if ((*line == 'n')
-		 && (!strncmp (line,    "news.newusers.questions", 23)
-		     || !strncmp (line, "news.groups.questions", 21)
-		     || !strncmp (line, "news.answers", 12)
-		     || !strncmp (line, "news.announce.newusers", 22)
-		     || !strncmp (line, "news.software.readers", 21)))
-	       {
-		  add_group (line, len, 0, 1);
-	       }
-	     else if ((*line == 'a')
-		      && (!strncmp (line, "alt.test", 8))
-		      && (line[8] <= ' '))
-	       {
-		  add_group (line, len, 0, 1);
-	       }
-	     else add_group (line, len, GROUP_UNSUBSCRIBED, 1);
+	     add_group (line, len, GROUP_UNSUBSCRIBED, 1);
 	  }
      }
+
+   if (create_flag)
+     {
+	char *name;
+	unsigned int i = 0;
+	Slrn_Group_Type *save = Slrn_Group_Current_Group, *last = NULL;
+	
+	if (OK_GROUPS == Slrn_Server_Obj->sv_list ("SUBSCRIPTIONS"))
+	  {
+	     name = line;
+	     if (Slrn_Server_Obj->sv_read_line (line, sizeof (line)) <= 0)
+	       name = NULL;
+	  }
+	else
+	  name = *Subscriptions;
+	
+	while (name != NULL)
+	  {
+	     if (NULL != (Slrn_Group_Current_Group = find_group_entry (name, strlen (name))))
+	       {
+		  Slrn_Group_Current_Group->flags &= ~GROUP_HIDDEN;
+		  Slrn_Group_Current_Group->flags &= ~GROUP_UNSUBSCRIBED;
+		  last = place_group_in_newsrc_order (last);
+	       }
+	     
+	     if (name != line)
+	       name = Subscriptions[++i];
+	     else if (Slrn_Server_Obj->sv_read_line (line, sizeof (line)) <= 0)
+	       name = NULL;
+	  }
+	
+	Slrn_Group_Current_Group = save;
+	Slrn_Groups_Dirty = 1;
+	Slrn_Write_Newsrc_Flags = 0;
+     }
+
+   Kill_After_Max = 0;
 }
 
 /*}}}*/
@@ -2383,10 +2489,9 @@ static int read_and_parse_newsrc_file (void)
    char *vline;
    unsigned int vlen;
    char file[SLRN_MAX_PATH_LEN];
-   Slrn_Group_Type *last_group = insert_new_groups ();
+   Slrn_Group_Type *last_group = NULL;
 
-   if ((NULL == (vp = slrn_open_home_vfile (Slrn_Newsrc_File, file,
-					    sizeof (file))))
+   if ((NULL == (vp = vopen (Slrn_Newsrc_File, 4096, 0)))
        && (NULL == (vp = slrn_open_home_vfile (".newsrc", file,
 					       sizeof (file)))))
      slrn_exit_error ("Unable to open %s.", file);
@@ -2431,17 +2536,19 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
    if (create_flag)
      {
 	FILE *fp;
-	char file[SLRN_MAX_PATH_LEN];
 
 	/* See if we can open the file. */
-	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "w", file,
-					       sizeof (file), 1)))
+#ifdef VMS
+	if (NULL == (fp = fopen (Slrn_Newsrc_File, "w", "fop=cif")))
+#else
+	if (NULL == (fp = fopen (Slrn_Newsrc_File, "w")))
+#endif
 	  {
-	     slrn_exit_error ("Unable to create %s.", file);
+	     slrn_exit_error ("Unable to create %s.", Slrn_Newsrc_File);
 	  }
 	fclose (fp);
 	fputs ("\n--The next step may take a while if the NNTP connection is slow.--\n\n", stdout);
-	fprintf (stdout, "Creating %s.", file);
+	fprintf (stdout, "Creating %s.", Slrn_Newsrc_File);
 	fflush (stdout);
      }
    
@@ -2454,6 +2561,8 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
        && (-1 == read_and_parse_newsrc_file ()))
      slrn_exit_error ("Unable to read newsrc file");
    
+   insert_new_groups ();
+   
    Slrn_Group_Current_Group = Groups;
 
    init_group_win_struct ();
@@ -2465,8 +2574,8 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
     * so that they will become invisble again.
     */
    Slrn_Group_Current_Group = Groups;
-   while ((Slrn_Group_Current_Group != NULL)
-	  && (Slrn_Group_Current_Group->flags & GROUP_NEW_GROUP_FLAG))
+   while ((Slrn_Group_Current_Group != NULL) &&
+	  (create_flag || (Slrn_Group_Current_Group->flags & GROUP_NEW_GROUP_FLAG)))
      {
 	Slrn_Group_Current_Group->flags &= ~GROUP_HIDDEN;
 	Slrn_Group_Current_Group = Slrn_Group_Current_Group->next;
@@ -2479,23 +2588,6 @@ int slrn_read_newsrc (int create_flag) /*{{{*/
    
    find_line_num ();
    
-   if (create_flag)
-     {
-	if (Group_Window.num_lines == 0)
-	  {
-	     Slrn_Group_Current_Group = Groups;
-	     while (Slrn_Group_Current_Group != NULL)
-	       {
-		  Slrn_Group_Current_Group->flags &= ~GROUP_HIDDEN;
-		  Slrn_Group_Current_Group = Slrn_Group_Current_Group->next;
-	       }
-	     Slrn_Group_Current_Group = Groups;
-	     find_line_num ();
-	  }
-	Slrn_Groups_Dirty = 1;
-	Slrn_Write_Newsrc_Flags = 0;
-     }
-   
    group_bob ();
    return 0;
 }
@@ -2506,7 +2598,7 @@ int slrn_write_newsrc (void) /*{{{*/
 {
    Slrn_Group_Type *g;
    Slrn_Range_Type *r;
-   char file[SLRN_MAX_PATH_LEN], backup_file[SLRN_MAX_PATH_LEN];
+   char backup_file[SLRN_MAX_PATH_LEN];
    static FILE *fp;
    int pass;
    int max;
@@ -2529,39 +2621,35 @@ int slrn_write_newsrc (void) /*{{{*/
    
    slrn_message_now ("Writing %s ...", Slrn_Newsrc_File);
    
-   slrn_make_home_filename (Slrn_Newsrc_File, file, sizeof (file));
-   
    /* Try to preserve .newsrc permissions and owner/group.  This also 
     * confirms existence of file.
     */
-   stat_worked = (-1 != stat (file, &filestat));
+   stat_worked = (-1 != stat (Slrn_Newsrc_File, &filestat));
   
 #ifdef VMS
-   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak", file);
+   slrn_snprintf (backup_file, sizeof (backup_file), "%s-bak", Slrn_Newsrc_File);
 #else
 # ifdef SLRN_USE_OS2_FAT
-   slrn_os2_make_fat (backup_file, sizeof (backup_file), file, ".bak");
+   slrn_os2_make_fat (backup_file, sizeof (backup_file), Slrn_Newsrc_File, ".bak");
 # else
-   slrn_snprintf (backup_file, sizeof (backup_file), "%s~", file);
+   slrn_snprintf (backup_file, sizeof (backup_file), "%s~", Slrn_Newsrc_File);
 # endif
 #endif
    
    /* Create a temp backup file.  Delete it later if user 
     * does not want backups.
     */
-   (void) slrn_delete_file (backup_file);
-
    have_backup = 1;
-   if (-1 == rename (file, backup_file))
+   if (-1 == rename (Slrn_Newsrc_File, backup_file))
      {
 	have_backup = 0;
      }
    
    
-   if (NULL == (fp = fopen (file, "w")))
+   if (NULL == (fp = fopen (Slrn_Newsrc_File, "w")))
      {
-	slrn_error ("Unable to open file %s for writing.", file);
-	if (have_backup) (void) rename (backup_file, file);
+	slrn_error ("Unable to open file %s for writing.", Slrn_Newsrc_File);
+	if (have_backup) (void) rename (backup_file, Slrn_Newsrc_File);
 	slrn_init_hangup_signals (1);
 	return -1;
      }
@@ -2576,10 +2664,10 @@ int slrn_write_newsrc (void) /*{{{*/
 #  endif
    if (stat_worked)
      {
- 	if (-1 == chmod (file, filestat.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)))
-	  (void) chmod (file, S_IWUSR | S_IRUSR);
+	if (-1 == chmod (Slrn_Newsrc_File, filestat.st_mode & (S_IRUSR | S_IWUSR | S_IXUSR)))
+	  (void) chmod (Slrn_Newsrc_File, S_IWUSR | S_IRUSR);
 
- 	(void) chown (file, filestat.st_uid, filestat.st_gid);
+ 	(void) chown (Slrn_Newsrc_File, filestat.st_uid, filestat.st_gid);
      }
 # endif
 #endif
@@ -2702,8 +2790,7 @@ int slrn_write_newsrc (void) /*{{{*/
    if (stat_worked)
      {
 	/* Put back orginal file */
-	(void) slrn_delete_file (file);
-	if (have_backup) (void) rename (backup_file, file);
+	if (have_backup) (void) rename (backup_file, Slrn_Newsrc_File);
      }
    
    slrn_init_hangup_signals (1);
@@ -2729,9 +2816,107 @@ static void group_quick_help (void) /*{{{*/
 }
 
 /*}}}*/
+
+static char *group_display_format_cb (char ch, void *data, int *len, int *color) /*{{{*/
+{
+   Slrn_Group_Type *g = (Slrn_Group_Type*) data;
+   static char buf[32];
+   char *retval = buf;
+   
+   *retval = 0;
+   if (g == NULL) return retval;
+   
+   switch (ch)
+     {
+      case 'F':
+	if (g->flags & GROUP_UNSUBSCRIBED) retval = "U";
+	else if (g->flags & GROUP_NEW_GROUP_FLAG) retval = "N";
+	else retval = " ";
+	*len = 1;
+	break;
+      case 'd':
+	if (NULL == (retval = g->descript))
+	  retval = " ";
+	if (color != NULL) *color = GROUP_DESCR_COLOR;
+	break;
+      case 'h':
+	if (g->range.max == -1)
+	  {
+	     retval = "?";
+	     *len = 1;
+	  }
+	else
+	  *len = sprintf (buf, "%d", g->range.max); /* safe */
+	break;
+      case 'l':
+	if (g->range.min == -1)
+	  {
+	     retval = "?";
+	     *len = 1;
+	  }
+	else
+	  *len = sprintf (buf, "%d", g->range.min); /* safe */
+	break;
+      case 'n':
+	retval = g->group_name;
+	if (color != NULL) *color = GROUP_COLOR;
+	break;
+      case 't':
+	if (g->range.max == -1)
+	  {
+	     retval = "?";
+	     *len = 1;
+	  }
+	else
+	  *len = sprintf (buf, "%d", g->range.max - g->range.min + 1); /* safe */
+	break;
+      case 'u':
+	*len = sprintf (buf, "%d", g->unread); /* safe */
+	break;
+     }
+   
+   return retval;
+}
+/*}}}*/
+
+static char *group_status_line_cb (char ch, void *data, int *len, int *color) /*{{{*/
+{
+   static char buf[66];
+   char *retval = NULL;
+   
+   (void) data; (void) color; /* we currently don't use these */
+   
+   switch (ch)
+     {
+      case 'L':
+	retval = slrn_print_percent (buf, &Group_Window, 1);
+	break;
+      case 'P':
+	retval = slrn_print_percent (buf, &Group_Window, 0);
+	break;
+      case 'D':
+	retval = Slrn_Groups_Dirty ? "*" : " ";
+	*len = 1;
+	break;
+      case 's':
+	retval = Slrn_Server_Obj->sv_name;
+	break;
+      default:
+	if (Slrn_Group_Current_Group != NULL)
+	  retval = group_display_format_cb (ch,
+					    (void *) Slrn_Group_Current_Group,
+					    len, NULL);
+	break;
+     }
+   
+   return retval;
+}
+/*}}}*/
+
 static void group_update_screen (void) /*{{{*/
 {
    Slrn_Group_Type *g;
+   char *fmt = Group_Display_Formats[Group_Format_Number];
    int height = (int) Group_Window.nrows;
    int row;
    
@@ -2750,6 +2935,9 @@ static void group_update_screen (void) /*{{{*/
 	Slrn_Full_Screen_Update = 1;
 	g = (Slrn_Group_Type *) Group_Window.top_window_line;
      }
+   
+   if ((fmt == NULL) || (*fmt == 0))
+     fmt = "  %F%-5u  %n%45g%d";
 
    for (row = 0; row < height; row++)
      {	
@@ -2760,37 +2948,8 @@ static void group_update_screen (void) /*{{{*/
 	  {
 	     if (Slrn_Full_Screen_Update || (g->flags & GROUP_TOUCHED))
 	       {
-		  SLsmg_gotorc (row + 1, 0);
-		  SLsmg_printf ("  %c%5d  ",
-				((g->flags & GROUP_UNSUBSCRIBED) ? 'U'
-				 : ((g->flags & GROUP_NEW_GROUP_FLAG) ? 'N' : ' ')),
-				g->unread);
-		  slrn_set_color (GROUP_COLOR);
-		  SLsmg_printf ("%s", g->group_name);
-		  slrn_set_color (0);
-		  SLsmg_erase_eol ();
-		  if (Slrn_Group_Display_Descriptions)
-		    {
-		       if (g->descript != NULL)
-			 {
-			    int dsc_row;
-			    
-			    dsc_row = SLsmg_get_column ();
-			    
-			    if (dsc_row < Slrn_Group_Description_Column)
-			      dsc_row = Slrn_Group_Description_Column;
-			    else dsc_row++;
-			    SLsmg_gotorc (row + 1, dsc_row);
-			    SLsmg_set_color (GROUP_DESCR_COLOR);
-			    SLsmg_write_string (g->descript);
-			    SLsmg_set_color (0);
-			 }
-		    }
-		  else if (g->range.max != -1)
-		    {
-		       SLsmg_gotorc (row + 1, 63);
-		       SLsmg_printf ("%7d-%-7d", g->range.min, g->range.max);
-		    }
+		  slrn_custom_printf (fmt, group_display_format_cb,
+				      (void *) g, row + 1, 0);
 		  g->flags &= ~GROUP_TOUCHED;
 	       }
 	     g = g->next;
@@ -2801,17 +2960,12 @@ static void group_update_screen (void) /*{{{*/
 	     SLsmg_erase_eol ();
 	  }
      }
-
-   SLsmg_gotorc (SLtt_Screen_Rows - 2, 0);
-   slrn_set_color (STATUS_COLOR);
-   if (Slrn_Groups_Dirty)
-     SLsmg_write_string ("-*-News Groups: ");
-   else SLsmg_write_string ("---News Groups: ");
    
-   if (Slrn_Server_Obj->sv_name != NULL)
-     SLsmg_write_string (Slrn_Server_Obj->sv_name);
-
-   slrn_print_percent (SLtt_Screen_Rows - 2, 60, &Group_Window);
+   fmt = Slrn_Group_Status_Line;
+   if ((fmt == NULL) || (*fmt == 0))
+     fmt = "-%D-News Groups: %s %-20g -- %L (%P)";
+   slrn_custom_printf (fmt, group_status_line_cb, NULL, SLtt_Screen_Rows - 2,
+		       STATUS_COLOR);
    
    if (Slrn_Use_Mouse) slrn_update_group_menu ();
    else slrn_update_top_status_line ();
@@ -2835,3 +2989,106 @@ static void group_update_screen (void) /*{{{*/
 }
 
 /*}}}*/
+
+/* intrinsic functions */
+#if SLRN_HAS_SLANG
+void slrn_intr_get_group_order (void) /*{{{*/
+{
+# if SLANG_VERSION < 10400
+   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+# else
+   Slrn_Group_Type *g;
+   SLang_Array_Type *retval;
+   int n = 0;
+   
+   g = Groups;
+   while (g != NULL)
+     {
+	n++;
+	g = g->next;
+     }
+   
+   retval = SLang_create_array (SLANG_STRING_TYPE, 0, NULL, &n, 1);
+   if (retval == NULL)
+     return;
+   
+   n = 0;
+   g = Groups;
+   while (g != NULL)
+     {
+	if (-1 == SLang_set_array_element (retval, &n, &g->group_name))
+	  {
+	     SLang_free_array (retval);
+	     return;
+	  }
+	n++;
+	g = g->next;
+     }
+   
+   (void) SLang_push_array (retval, 1);
+# endif /* SLANG_VERSION */
+}
+/*}}}*/
+
+void slrn_intr_set_group_order (void) /*{{{*/
+{
+#if SLANG_VERSION < 10400
+   slrn_error ("To use this feature, please update s-lang and recompile slrn.");
+# else
+   SLang_Array_Type *at;
+   Slrn_Group_Type *last = NULL, *rest = Groups;
+   int i, rows;
+   
+   if (-1 == SLang_pop_array_of_type (&at, SLANG_STRING_TYPE))
+     {
+	slrn_error ("Array of string expected.");
+	return;
+     }
+   
+   if (at->num_dims != 1)
+     {
+	slrn_error ("One-dimensional array expected.");
+	SLang_free_array (at);
+	return;
+     }
+   
+   rows = at->dims[0];
+   
+   for (i = 0; i < rows; i++)
+     {
+	char *name;
+	Slrn_Group_Type *g;
+	
+	(void) SLang_get_array_element (at, &i, &name);
+	
+	if (NULL == (g = find_group_entry (name, strlen (name))))
+	  continue;
+	
+	if (g->next != NULL)
+	  g->next->prev = g->prev;
+	if (g->prev != NULL)
+	  g->prev->next = g->next;
+	g->prev = last;
+	if (last == NULL)
+	  Groups = g;
+	else
+	  last->next = g;
+	if (g == rest)
+	  rest = rest->next;
+	last = g;
+     }
+   
+   if (last != NULL)
+     last->next = rest;
+   if (rest != NULL)
+     rest->prev = last;
+   
+   SLang_free_array (at);
+   
+   find_line_num ();
+   Slrn_Full_Screen_Update = 1;
+   Slrn_Groups_Dirty = 1;
+# endif /* SLANG_VERSION */
+}
+/*}}}*/
+#endif /* SLRN_HAS_SLANG */
