@@ -45,7 +45,7 @@
 #include "sltcp.h"
 #include "util.h"
 #include "server.h"
-
+#include "snprintf.h"
 
 
 #define GL_MAX_RESPONSE_LINE_LEN	1024
@@ -405,7 +405,7 @@ static int connect_to_gl_host (GL_Type *gl, char *cmd) /*{{{*/
 	return -1;
      }
    
-   if (NULL == (gl->tcp = sltcp_open_connection (host, port)))
+   if (NULL == (gl->tcp = sltcp_open_connection (host, port, 0)))
      return -1;
    
    /* We should be able to read OK ...  If not, we failed to make proper 
@@ -698,14 +698,17 @@ static int gl_open_ratings (GL_Type *gl, char *group) /*{{{*/
 
 static int gl_put_rating (GL_Type *gl, GL_Rating_Type *rt) /*{{{*/
 {
-   char buf [GL_MAX_RESPONSE_LINE_LEN];
+   char *buf;
+   int retval;
    
    if (rt->rating <= 0) return 0;
-   sprintf (buf, "%s :rating=%4.2f :sawHeader=1", rt->msgid, (float) rt->rating);
-   if (rt->saw_article)
-     strcat (buf + strlen (buf), " :sawArticle=1");
+   buf = slrn_strdup_printf ("%s :rating=%4.2f :sawHeader=1%s",
+			     rt->msgid, (float) rt->rating,
+			     rt->saw_article ? " :sawArticle=1" : "");
+   retval = send_gl_line (gl, buf, 0);
+   slrn_free (buf);
    
-   return send_gl_line (gl, buf, 0);
+   return retval;
 }
 
 /*}}}*/
@@ -972,10 +975,10 @@ static int read_grplens_file (void) /*{{{*/
    unsigned int linenum;
    GL_Config_File_Type *cfg;
 
-   fp = slrn_open_home_file (".grouplens", "r", file, 0);
+   fp = slrn_open_home_file (".grouplens", "r", file, sizeof (file), 0);
    if (fp == NULL) 
      {
-	fp = slrn_open_home_file (".grplens", "r", file, 0);
+	fp = slrn_open_home_file (".grplens", "r", file, sizeof (file), 0);
 	if (fp == NULL)
 	  return -1;
      }
