@@ -45,21 +45,17 @@
 #include "art.h"
 #include "util.h"
 #include "snprintf.h"
-
-#if SLRN_HAS_MIME
-/* rest of file in this ifdef */
-
 #include "mime.h"
 
-int Slrn_Use_Mime = 1;
+#if ! SLRN_HAS_MIME
+int Slrn_Use_Mime = 0;
+#else /* rest of file in this ifdef */
+
+int Slrn_Use_Mime = 5;
 int Slrn_Use_Meta_Mail = 1;
 char *Slrn_MetaMail_Cmd;
 char *Slrn_Utf8_Table = NULL;
 
-int Slrn_Mime_Was_Parsed;
-int Slrn_Mime_Was_Modified;
-int Slrn_Mime_Needs_Metamail;
-
 char *Slrn_Mime_Display_Charset;
 
 /* These are all supersets of US-ASCII.  Only the first N characters are 
@@ -130,6 +126,10 @@ int slrn_set_compatible_charsets (char *charsets)
    slrn_free (buf);
    buf = NULL;
    slrn_free ((char *) Custom_Compatible_Charsets);
+   Custom_Compatible_Charsets = NULL;
+   
+   if (*charsets == 0)
+     return 0;
 
    n = 1;
    p = charsets;
@@ -151,7 +151,7 @@ int slrn_set_compatible_charsets (char *charsets)
      }
    
    pp = Custom_Compatible_Charsets;
-   p = charsets;
+   p = buf;
    while (n-- != 0)
      {
 	*pp++ = p;
@@ -289,7 +289,6 @@ static int parse_content_type_line (Slrn_Article_Type *a)
 	     len = b - charset;
 	     
 	     Char_Set = find_compatible_charset (charset, len);
-	     if (Char_Set == NULL) return -1;
 	     return 0;
 	  }
 	line = line->next;
@@ -350,25 +349,32 @@ static int Index_Hex[128] =
 
 static char *decode_quoted_printable (char *dest,
 				      char *src, char *srcmax,
-				      int treat_underscore_as_space)
+				      int treat_underscore_as_space,
+				      int strip_8bit)
 {
    char *allowed_in_qp = "0123456789ABCDEFabcdef";
-   char ch;
+   unsigned char ch, mask = 0x0;
+#ifndef SLRNPULL_CODE
+   if (strip_8bit && (NULL == Char_Set))
+     mask = 0x80;
+#endif
    while (src < srcmax)
      {
-	ch = *src++;
+	ch = (unsigned char) *src++;
 	if ((ch == '=') && (src + 1 < srcmax)
 	    && (NULL != slrn_strchr (allowed_in_qp, src[0]))
 	    && (NULL != slrn_strchr (allowed_in_qp, src[1])))
 	  {
-	     *dest++ = (16 * HEX(src[0])) + HEX(src[1]);
+	     ch = (16 * HEX(src[0])) + HEX(src[1]);
+	     if (ch & mask) ch = '?';
+	     *dest++ = (char) ch;
 	     src += 2;
 	  }
 	else if ((ch == '_') && treat_underscore_as_space)
 	  {
 	     *dest++ = ' ';
 	  }
-	else *dest++ = ch;
+	else *dest++ = (char) ch;
      }
    return dest;
 }
@@ -579,7 +585,7 @@ int slrn_rfc1522_decode_string (char *s)
 	
 	if (method == 'B')
 	  s1 = decode_base64 (s1, txt, s);
-	else s1 = decode_quoted_printable (s1, txt, s, 1);
+	else s1 = decode_quoted_printable (s1, txt, s, 1, 0);
 	
 	if (slrn_case_strncmp((unsigned char *)"utf-8",
 			      (unsigned char *)charset, 5) == 0)
@@ -617,7 +623,7 @@ static void rfc1522_decode_headers (Slrn_Article_Type *a)
 	if (slrn_rfc1522_decode_string (line->buf))
 	  {
 	     a->is_modified = 1;
-	     Slrn_Mime_Was_Modified = 1;
+	     a->mime_was_modified = 1;
 	  }
 	line = line->next;
      }
@@ -669,6 +675,16 @@ static void decode_mime_base64 (Slrn_Article_Type *a)
    buf_pos = decode_base64(buf_dest, buf_src, buf_src+len);
    *buf_pos = '\0';
    
+   if (Char_Set == NULL)
+     {
+	buf_pos = buf_dest;
+	while (*buf_pos)
+	  {
+	     if (*buf_pos & 0x80) *buf_pos = '?';
+	     buf_pos++;
+	  }
+     }
+   
    l = body_start;
    body_start = body_start->prev;
    
@@ -714,7 +730,7 @@ static void decode_mime_base64 (Slrn_Article_Type *a)
    slrn_free(base);
    
    a->is_modified = 1;
-   Slrn_Mime_Was_Modified = 1;
+   a->mime_was_modified = 1;
 }
 
 /* This function checks if the last character on curr_line is an = and 
@@ -798,12 +814,12 @@ static void decode_mime_quoted_printable (Slrn_Article_Type *a)
 	     len = strlen (b);
 	  }
 
-	b = decode_quoted_printable (b, b, b + len, 0);
+	b = decode_quoted_printable (b, b, b + len, 0, 1);
 	if (b < line->buf + len)
 	  {
 	     *b = 0;
 	     a->is_modified = 1;
-	     Slrn_Mime_Was_Modified = 1;
+	     a->mime_was_modified = 1;
 	  }
 	
 	line = line->next;
@@ -838,28 +854,26 @@ static void decode_mime_utf8 (Slrn_Article_Type *a)
 	  {
 	     *b = 0;
 	     a->is_modified = 1;
-	     Slrn_Mime_Was_Modified = 1;
+	     a->mime_was_modified = 1;
 	  }
 	
 	line = line->next;
      }
 }
 
-void slrn_mime_article_init (void)
+void slrn_mime_article_init (Slrn_Article_Type *a)
 {
-   Slrn_Mime_Was_Modified = 0;
-   Slrn_Mime_Was_Parsed = 0;
-   Slrn_Mime_Needs_Metamail = 0;
+   a->mime_was_modified = 0;
+   a->mime_was_parsed = 0;
+   a->mime_needs_metamail = 0;
 }
 
 void slrn_mime_process_article (Slrn_Article_Type *a)
 {
-   if ((Slrn_Use_Mime == 0)
-       || Slrn_Mime_Was_Parsed
-       || (a == NULL))
+   if ((a == NULL) || (a->mime_was_parsed))
      return;
 
-   Slrn_Mime_Was_Parsed = 1;	       /* or will be */
+   a->mime_was_parsed = 1;	       /* or will be */
    
    rfc1522_decode_headers (a);
 
@@ -867,7 +881,7 @@ void slrn_mime_process_article (Slrn_Article_Type *a)
    if ((-1 == parse_content_type_line (a))
        || (-1 == parse_content_transfer_encoding_line (a)))
      {
-	Slrn_Mime_Needs_Metamail = 1;
+	a->mime_needs_metamail = 1;
 	return;
      }
    
@@ -888,11 +902,12 @@ void slrn_mime_process_article (Slrn_Article_Type *a)
 	break;
 	
       default:
-	Slrn_Mime_Needs_Metamail = 1;
+	a->mime_needs_metamail = 1;
 	return;
      }
    
-   if ((Slrn_Mime_Needs_Metamail == 0) &&
+   if ((a->mime_needs_metamail == 0) &&
+       (Char_Set != NULL) &&
        (slrn_case_strncmp((unsigned char *)"utf-8",
 			  (unsigned char *)Char_Set, 5) == 0))
      decode_mime_utf8 (a);
