@@ -106,10 +106,7 @@ extern int h_errno;
 #include "art.h"
 #include "chmap.h"
 #include "snprintf.h"
-
-#if SLRN_HAS_MIME
 #include "mime.h"
-#endif
 
 #ifdef VMS
 /* valid filname chars for unix equiv of vms filename */
@@ -195,169 +192,83 @@ void slrn_redraw (void) /*{{{*/
 
 /*}}}*/
 
-void slrn_print_percent (int row, int col, SLscroll_Window_Type *w) /*{{{*/
+/* theoretically, buf needs at least space for 66 characters */
+char *slrn_print_percent (char *buf, SLscroll_Window_Type *w, int lines) /*{{{*/
 {
-   int bot_showing;
-   unsigned int bot_number;
-   
-   SLsmg_erase_eol ();
-   SLsmg_gotorc (row, col);
-   SLsmg_printf ("-- %d/%d", w->line_num, w->num_lines);
-
-   bot_number = w->line_num + (w->nrows - w->window_row) - 1;
-   
-   bot_showing = ((w->bot_window_line == NULL)
-		  || (w->num_lines == bot_number));
-
-   if (w->line_num == w->window_row + 1)
+   if (lines)
      {
-	SLsmg_write_string (bot_showing ? "  (All)" : "  (Top)" );
+	sprintf (buf, "%d/%d", w->line_num, w->num_lines);
      }
-   else if (bot_showing) SLsmg_write_string("  (Bot)");
-   else SLsmg_printf("  (%d%%)", (100 * bot_number) / w->num_lines);
-   SLsmg_erase_eol ();
-}
-
+   else
+     {
+	int bot_showing;
+	unsigned int bot_number;
+	
+	bot_number = w->line_num + (w->nrows - w->window_row) - 1;
+	bot_showing = ((w->bot_window_line == NULL)
+		       || (w->num_lines == bot_number));
+	
+	if (w->line_num == w->window_row + 1)
+	  strcpy (buf, bot_showing ? "All" : "Top" ); /* safe */
+	else if (bot_showing)
+	  strcpy (buf, "Bot"); /* safe */
+	else
+	  sprintf (buf, "%d%%", (100 * bot_number) / w->num_lines); /* safe */
+    }
+  return buf;
+}   
 /*}}}*/
 
+static char *top_status_line_cb (char ch, void *data, int *len, int *color) /*{{{*/
+{
+   static char buf[32];
+   char *retval = NULL;
+   time_t now;
+   
+   (void) data; (void) color; /* we currently don't use these */
+   
+   switch (ch)
+     {
+      case 'd':
+	time(&now);
+	if (0 != (*len = strftime (buf, sizeof(buf), "%x", localtime(&now))))
+	  retval = buf;
+	break;
+	
+      case 'n':
+	if (NULL != Slrn_Group_Current_Group)
+	  retval = Slrn_Group_Current_Group->group_name;
+	break;
+	
+      case 's':
+	retval = Slrn_Server_Obj->sv_name;
+	break;
+	
+      case 't':
+	time(&now);
+	if (0 != (*len = strftime (buf, sizeof(buf), "%X", localtime(&now))))
+	  retval = buf;
+	break;
+	
+      case 'v':
+	retval = Slrn_Version;
+	break;
+	
+     }
+   return retval == NULL ? "" : retval;
+}
+/*}}}*/
 void slrn_update_top_status_line (void) /*{{{*/
 {
-   char *fmt;
-   char ch;
+   char *fmt = Slrn_Top_Status_Line;
    
    if (Slrn_Full_Screen_Update == 0) return;
    
-   fmt = Slrn_Top_Status_Line;
-   if (fmt == NULL)
+   if ((fmt == NULL) || (*fmt == 0))
      fmt = "slrn %v ** Press '?' for help, 'q' to quit. ** Server: %s";
-
-   SLsmg_gotorc (0, 0);
-   slrn_set_color (MENU_COLOR);
    
-   while ((ch = *fmt) != 0)
-     {
-	char *s;
-	int len;
-	int field_len;
-	char buf[32];
-	int right_justify;
-	int spaces = 0;
-	time_t now;
-
-	fmt++;
-	if (ch != '%')
-	  {
-	     SLsmg_write_nchars (&ch, 1);
-	     continue;
-	  }
-
-	ch = *fmt++;
-
-	s = NULL;
-	len = -1;
-
-	field_len = -1;
-	if (ch == '-')
-	  {
-	     right_justify = 1;
-	     ch = *fmt++;
-	  }
-	else
-	  right_justify = 0;
-
-	if (isdigit (ch))
-	  {
-	     field_len = 0;
-	     do
-	       {
-		  field_len = 10 * field_len + (int) (ch - '0');
-		  ch = *fmt++;
-	       }
-	     while (isdigit (ch));
-	  }
-
-	switch (ch)
-	  {
-	   case 0:
-	     slrn_set_color (0);
-	     SLsmg_erase_eol ();
-	     return;
-	     
-	   default:
-	     break;
-	     
-	   case '%':
-	     s = "%";
-	     len = 1;
-	     break;
-	     
-	   case 'd':
-	     time(&now);
-	     len = strftime(buf, sizeof(buf), "%x", localtime(&now));
-	     if (len == 0) s = NULL;
-	     else s = buf;
-	     break;
-	     
-	   case 'n':
-	     if (NULL != Slrn_Group_Current_Group)
-	       s = Slrn_Group_Current_Group->group_name;
-	     break;
-	     
-	   case 's':
-	     s = Slrn_Server_Obj->sv_name;
-	     break;
-	     
-	   case 't':
-	     time(&now);
-	     len = strftime(buf, sizeof(buf), "%X", localtime(&now));
-	     if (len == 0) s = NULL;
-	     else s = buf;
-	     break;
-	     
-	   case 'v':
-	     s = Slrn_Version;
-	     break;
-	     
-           case 'g':
-             SLsmg_erase_eol ();
-             if (right_justify)
-               field_len = SLtt_Screen_Cols - field_len;
-             SLsmg_gotorc (0, field_len);
-             break;
-	  }
-
-	if (s == NULL)
-	  continue;
-
-	if (len == -1)
-	  len = strlen (s);
-	if (field_len != -1)
-	  {
-	     if (field_len > len)
-	       spaces = field_len - len;
-	     else 
-	       len = field_len;
-	  }
-	if (right_justify)
-	  {
-	     while (spaces)
-	       {
-		  SLsmg_write_nchars (" ", 1);
-		  spaces--;
-	       }
-	  }
-	SLsmg_write_nchars (s, len);
-	while (spaces)
-	  {
-	     spaces--;
-	     SLsmg_write_nchars (" ", 1);
-	  }
-     }
-
-   SLsmg_erase_eol ();
-   slrn_set_color (0);
+   slrn_custom_printf (fmt, top_status_line_cb, NULL, 0, MENU_COLOR);
 }
-
 /*}}}*/
 
 /*}}}*/
@@ -526,6 +437,237 @@ void slrn_error_now (unsigned secs, char *fmt, ...) /*{{{*/
 
 /*}}}*/
 
+/* Wrapper around the SLsmg routine. */
+void slrn_write_nchars (char *s, unsigned int n) /*{{{*/
+{
+   unsigned char *s1, *smax;
+   unsigned int eight_bit;
+
+   s1 = (unsigned char *) s;
+   smax = s1 + n;
+   eight_bit = SLsmg_Display_Eight_Bit;
+   
+   while (s1 < smax)
+     {
+	if ((*s1 & 0x80) && (eight_bit > (unsigned int) *s1))
+	  {
+	     if (s != (char *) s1)
+	       SLsmg_write_nchars (s, (unsigned int) ((char *)s1 - s));
+	     SLsmg_write_char ('?');
+	     s1++;
+	     s = (char *) s1;
+	  }
+	else s1++;
+     }
+   
+   if (s != (char *)s1)
+     SLsmg_write_nchars (s, (unsigned int) ((char *)s1 - s));
+}
+/*}}}*/
+
+/* generic function to display format strings;
+ * cb is called when a descriptor other than '%', '?' or 'g' is encountered */
+void slrn_custom_printf (char *fmt, PRINTF_CB cb, void *param, /*{{{*/
+			 int row, int def_color)
+{
+   char ch, *cond_end = NULL, *elsepart = NULL;
+   
+   SLsmg_gotorc (row, 0);
+   slrn_set_color (def_color);
+   
+   while ((ch = *fmt) != 0)
+     {
+	char *s;
+	int color = def_color;
+	int len = -1, field_len = -1;
+	int right_justify, spaces = 0;
+	
+	if (fmt == elsepart)
+	  {
+	     fmt = cond_end + 1; /* skip the '?' */
+	     elsepart = cond_end = NULL;
+	     continue;
+	  }
+	
+	if (ch != '%')
+	  {
+	     if ((NULL == (s = strchr (fmt, '%'))) && (elsepart == NULL))
+	       {
+		  SLsmg_write_string (fmt);
+		  break;
+	       }
+	     else
+	       {
+		  char *end;
+		  if ((elsepart != NULL) && (s > elsepart)) end = elsepart;
+		  else end = s;
+		  SLsmg_write_nchars (fmt, end - fmt);
+		  fmt = end;
+		  continue;
+	       }
+	  }
+	
+	fmt++;
+	ch = *fmt++;
+	
+	if (ch == '?')
+	  {
+	     int column;
+	     char *res;
+	     
+	     if (((ch = *fmt++) == '\0') || (*fmt++ != '?') ||
+		 (NULL == (cond_end = strchr (fmt, '?'))))
+	       break; /* syntax error; stop here */
+	     if ((NULL == (elsepart = strchr (fmt, '&'))) ||
+		 (elsepart > cond_end))
+	       elsepart = cond_end;
+	     
+	     column = SLsmg_get_column (); /* callback could print something */
+	     res = (cb)(ch, param, &len, &color);
+	     SLsmg_gotorc (row, column);
+	     
+	     if ((res == NULL) || (*res == '\0') ||
+		 ((*res == '0' || isspace(*res)) && *(res+1) == '\0'))
+	       {
+		  fmt = elsepart + 1;
+		  if (elsepart == cond_end) cond_end = NULL;
+		  elsepart = cond_end;
+	       }
+	     continue;
+	  }
+	
+	s = NULL;
+	
+	if (ch == '-')
+	  {
+	     right_justify = 1;
+	     ch = *fmt++;
+	  }
+	else
+	  right_justify = 0;
+	
+	if (isdigit (ch))
+	  {
+	     field_len = 0;
+	     do
+	       {
+		  field_len = 10 * field_len + (int) (ch - '0');
+		  ch = *fmt++;
+	       }
+	     while (isdigit (ch));
+	  }
+	
+	switch (ch)
+	  {
+	   case 0:
+	     break;
+
+	   case '%':
+	     s = "%";
+	     len = 1;
+	     break;
+	     
+	   case 'g':
+	     SLsmg_erase_eol ();
+	     if (right_justify)
+	       field_len = SLtt_Screen_Cols - field_len;
+	     SLsmg_gotorc (row, field_len);
+	     break;
+	     
+	   default:
+	     s = (cb)(ch, param, &len, &color);
+	     break;
+	  }
+	
+	if (s == NULL)
+	  continue;
+
+	if (color != def_color)
+	  slrn_set_color (color);
+
+	if (len == -1) len = strlen (s);
+	if (field_len != -1)
+	  {
+	     int i;
+	     
+	     if (field_len > len)
+	       spaces = field_len - len;
+	     else
+	       len = field_len;
+	     
+	     for (i = 0; i < len; i++)
+	       {
+		  /* S-Lang uses two characters to represent control chars */
+		  if ((unsigned char) s[i] < 0x20)
+		    {
+		       if (spaces) spaces--;
+		       else len--;
+		    }
+	       }
+	     if (i == len + 1) spaces = 1;
+	  }
+	if (right_justify)
+	  {
+	     while (spaces)
+	       {
+		  SLsmg_write_nchars (" ", 1);
+		  spaces--;
+	       }
+	  }
+        slrn_write_nchars (s, len);
+	while (spaces)
+	  {
+	     SLsmg_write_nchars (" ", 1);
+	     spaces--;
+	  }
+	if (color != def_color) 
+	  slrn_set_color (def_color);
+     }
+   
+   SLsmg_erase_eol ();
+}
+/*}}}*/
+
+int slrn_set_display_format (char **formats, unsigned int num, char *entry) /*{{{*/
+{
+   if (num >= SLRN_MAX_DISPLAY_FORMATS)
+     return -1;
+   
+   if (formats[num] != NULL)
+     SLang_free_slstring (formats[num]);
+
+   if ((entry == NULL) || (*entry == 0))
+     {
+	formats[num] = NULL;
+	return 0;
+     }
+
+   if (NULL == (formats[num] = SLang_create_slstring (entry)))
+     return -1;
+   
+   return 0;
+}
+/*}}}*/
+
+unsigned int slrn_toggle_format (char **formats, unsigned int cur) /*{{{*/
+{
+   unsigned int retval = cur;
+   
+   if (Slrn_Prefix_Arg_Ptr != NULL)
+     {
+	retval = (unsigned int) *Slrn_Prefix_Arg_Ptr % SLRN_MAX_DISPLAY_FORMATS;
+	Slrn_Prefix_Arg_Ptr = NULL;
+     }
+   else retval++;
+   
+   while ((retval != cur) && (formats[retval] == NULL))
+     retval = (retval + 1) % SLRN_MAX_DISPLAY_FORMATS;
+
+   Slrn_Full_Screen_Update = 1;
+   return retval;
+}
+/*}}}*/
+
 int slrn_check_batch (void)
 {
    if (Slrn_Batch == 0) return 0;
@@ -575,9 +717,13 @@ void slrn_make_home_filename (char *name, char *file, size_t n) /*{{{*/
    char *home;
 #ifndef VMS
    if (slrn_is_absolute_path (name)
-       || ((name[0] == '.') && (name[1] == SLRN_PATH_SLASH_CHAR))
+       || ((name[0] == '.') &&
+	   ((name[1] == SLRN_PATH_SLASH_CHAR) ||
+	    ((name[1] == '.') && (name[2] == SLRN_PATH_SLASH_CHAR))))
 #if defined(IBMPC_SYSTEM)
-       || ((name[0] == '.') && (name[1] == '/'))
+       || ((name[0] == '.') &&
+	   ((name[1] == '/') ||
+	    ((name[1] == '.') && name[2] == '/')))
 #endif
        )
      {
@@ -922,11 +1068,15 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
      }
    slrn_message_now ("Sending ...");
 
-   slrn_chmap_fix_file (file, 0);
+   if (Slrn_Use_Mime & MIME_ARCHIVE) /* else: do it later */
+     slrn_chmap_fix_file (file, 0);
    
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Replies_File, NULL))
      return -1;
-
+   
+   if ((Slrn_Use_Mime & MIME_ARCHIVE) == 0)
+     slrn_chmap_fix_file (file, 0);
+   
 #ifdef VMS
    buf = slrn_strdup_printf ("%s\"%s\"", MAIL_PROTOCOL, to);
    vms_send_mail (buf, subject, file);
@@ -938,7 +1088,7 @@ int slrn_mail_file (char *file, int edit, unsigned int editline, char *to, char
     * sendmail program.  This way I can strip out blank headers.
     */
 # if SLRN_HAS_MIME
-   if (Slrn_Use_Mime)
+   if (Slrn_Use_Mime & MIME_DISPLAY)
      {
 	FILE *fp, *pp;
 	int header = 1;
@@ -1787,6 +1937,20 @@ void slrn_get_mouse_rc (int *rp, int *cp) /*{{{*/
 
 /*}}}*/
 
+void slrn_evaluate_cmd (void) /*{{{*/
+{
+   char buf[256];
+   
+   *buf = '\0';
+   if (slrn_read_input ("S-Lang>", NULL, buf, 0, 0) > 0)
+     {
+	SLang_load_string (buf);
+     }
+   
+   SLang_Error = 0;
+}
+/*}}}*/
+
 /*}}}*/
 
 /*{{{ Misc Regexp Utility Functions */
