@@ -56,10 +56,7 @@
 #include "chmap.h"
 #include "menu.h"
 #include "version.h"
-
-#if SLRN_HAS_MIME
-# include "mime.h"
-#endif
+#include "mime.h"
 
 #define MAX_LINE_BUFLEN	2048
 
@@ -73,9 +70,9 @@ char *Slrn_Failed_Post_Filename;
 char *Slrn_Signoff_String;
 
 int Slrn_Reject_Long_Lines = 1;
+int Slrn_Netiquette_Warnings = 1;
 char *Slrn_Postpone_Dir;
 int Slrn_Generate_Message_Id = 1;
-int Slrn_Smart_Quote = 1;
 
 static int postpone_file (char *);
 
@@ -332,7 +329,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
      }
 
 # if SLRN_HAS_MIME
-   if (Slrn_Use_Mime) slrn_mime_add_headers (pp);
+   if (Slrn_Use_Mime & MIME_DISPLAY) slrn_mime_add_headers (pp);
 # endif
    
    fputs ("\n", pp);
@@ -348,7 +345,7 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
      ret = fputs ("\n", pp);
    
 # if SLRN_HAS_MIME
-   if (Slrn_Use_Mime) fp = slrn_mime_encode (fp);
+   if (Slrn_Use_Mime & MIME_DISPLAY) fp = slrn_mime_encode (fp);
 # endif
    
    while (NULL != fgets (line, sizeof (line) - 1, fp))
@@ -368,17 +365,20 @@ static int slrn_cc_file (char *file, char *to, char *msgid)
 }
 
 /* Returns -1 upon error, 0, if ok, 1 if needs repair, 2 is warning is issued */
+#define MAX_WARNINGS 10
 static int check_file_for_posting (char *file)
 {
-   char line[MAX_LINE_BUFLEN], *the_line;
+   char line[MAX_LINE_BUFLEN], buf[MAX_LINE_BUFLEN], *the_line;
    FILE *fp;
    unsigned int num;
-   char *err;
-   int newsgroups_found, subject_found, only_warning;
+   char *err, *warnings[MAX_WARNINGS];
+   int newsgroups_found, subject_found, followupto_found, warn;
    char ch;
    char *colon;
 
-   only_warning = newsgroups_found = subject_found = 0;
+   for (warn = 0; warn < MAX_WARNINGS; warn++)
+     warnings[warn] = NULL;
+   warn = newsgroups_found = followupto_found = subject_found = 0;
    err = NULL;
    fp = fopen (file, "r");
    
@@ -427,12 +427,29 @@ static int check_file_for_posting (char *file)
 	
 	if (!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "Newsgroups:", 11))
 	  {
+	     char *p = line;
 	     if (is_empty_header (line)) 
 	       {
 		  err = "The Newsgroups header is not allowed be to empty.";
 		  break;
 	       }
 	     newsgroups_found = 1;
+	     while (*(++p))
+	       {
+		  if (*p == ',') newsgroups_found++;
+	       }
+	     continue;
+	  }
+	
+	if ((!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "Followup-To:", 12))
+	    && (!is_empty_header (line)))
+	  {
+	     char *p = line;
+	     followupto_found = 1;
+	     while (*(++p))
+	       {
+		  if (*p == ',') followupto_found++;
+	       }
 	     continue;
 	  }
 	
@@ -451,12 +468,19 @@ static int check_file_for_posting (char *file)
 	     err = "This news reader will not accept user generated From lines.";
 	     break;
 	  }
+#endif
+#if ! SLRN_HAS_GEN_MSGID
+	if (!slrn_case_strncmp ((unsigned char *) line, (unsigned char *) "Message-Id:", 11))
+	  {
+	     err = "This news reader will not accept user generated Message-Ids.";
+	     break;
+	  }
 #endif
      }
 
    if (err == NULL) 
      {
-	if (subject_found == 0) 
+	if (subject_found == 0)
 	  {
 	     err = "Subject header is required.";
 	     num = 0;
@@ -466,6 +490,18 @@ static int check_file_for_posting (char *file)
 	     err = "Newsgroups header is required.";
 	     num = 0;
 	  }
+	else
+	  {
+	     if (newsgroups_found > 4)
+	       warnings[warn++ % MAX_WARNINGS] =
+	       "Please re-consider if posting to a large number of groups is necessary.";
+	     if (followupto_found > 1)
+	       warnings[warn++ % MAX_WARNINGS] =
+	       "In most cases, setting a \"Followup-To:\" multiple groups is a bad idea.";
+	     else if ((newsgroups_found > 1) && (followupto_found == 0))
+	       warnings[warn++ % MAX_WARNINGS] =
+	       "Setting a \"Followup-To:\" is recommended when crossposting.";
+	  }
      }
    
    /* Now body.  Check for non-quoted lines. */
@@ -473,6 +509,7 @@ static int check_file_for_posting (char *file)
      {
 	char *qs = Slrn_Quote_String;
 	unsigned int qlen;
+	int sig_lines = -1;
 #if SLRN_HAS_VERBATIM_MARKS
 	int is_verbatim = 0;
 #endif
@@ -483,7 +520,9 @@ static int check_file_for_posting (char *file)
 	the_line = NULL;
 	while (NULL != fgets (line, sizeof (line), fp))
 	  {
-	     num++;
+	     if (the_line == NULL) num++;
+	     if (sig_lines != -1) sig_lines++;
+	     
 	     if (!strncmp (line, qs, qlen))
 	       continue;
 #if SLRN_HAS_VERBATIM_MARKS
@@ -505,35 +544,44 @@ static int check_file_for_posting (char *file)
 	     if (is_verbatim)
 	       continue;
 #endif
+	     if (0 == strcmp (line, "-- \n"))
+	       {
+		  sig_lines = 0;
+		  continue;
+	       }
+	     
 	     colon = slrn_skip_whitespace (line);
 	     if (*colon == 0) continue;
 	     
 	     err = NULL;
-	     if (Slrn_Reject_Long_Lines == 0)
+	     
+	     if ((Slrn_Netiquette_Warnings == 0) &&
+		 (Slrn_Reject_Long_Lines == 0))
 	       break;
 	     
-	     if (strlen (line) > 81)   /* allow \n to slip */
+	     if ((the_line == NULL) && (strlen (line) > 81)) /* allow \n to slip */
 	       {
-		  err = "Line contains more than 80 characters.  You need to wrap it.";
-		  the_line = line;
-		  if (Slrn_Reject_Long_Lines == 2)
-		    only_warning = 1;
-		  break;
+		  the_line = buf; /* magic: slrn will know that we got here */
+		  strcpy (buf, line); /* safe */
 	       }
 	  }
+	if (sig_lines > 4)
+	  warnings[warn++ % MAX_WARNINGS] =
+	  "Please keep your signature short. 4 lines is a commonly accepted limit.";
      }
    
    fclose (fp);
 
-   if (err == NULL) return 0;
-   
-   if (Slrn_Batch)
+   if (Slrn_Batch && (err != NULL))
      {
 	slrn_error ("Message is not acceptable.");
-	if (err != NULL) slrn_error ("Reason: %s", err);
+	slrn_error ("Reason: %s", err);
 	return -1;
      }
-	  
+   
+   if ((err == NULL) && (!Slrn_Netiquette_Warnings || (*warnings == NULL)) &&
+       (!Slrn_Reject_Long_Lines || (the_line == NULL))) return 0;
+   
    Slrn_Full_Screen_Update = 1;
 
    slrn_set_color (0);
@@ -541,26 +589,59 @@ static int check_file_for_posting (char *file)
    SLsmg_cls ();
    SLsmg_gotorc (2,0);
    slrn_set_color (SUBJECT_COLOR);
-   SLsmg_write_string ("Your message is not acceptable for the following reason:");
-   slrn_set_color (ERROR_COLOR);
-   SLsmg_gotorc (4,8); SLsmg_write_string (err);
-   if (num && (the_line != NULL))
+   if (err != NULL)
      {
-	SLsmg_gotorc (6,0);
-	slrn_set_color (SUBJECT_COLOR);
-	SLsmg_printf ("This message was generated while looking at line %d:", num);
-	SLsmg_gotorc (8,0);
-	slrn_set_color (QUOTE_COLOR);
-	SLsmg_write_string (the_line);
-	SLsmg_gotorc (12, 0);
-	slrn_set_color (ARTICLE_COLOR);
-	SLsmg_write_string ("Perhaps this error was generated because you did not separate the header");
-	SLsmg_gotorc (13, 0);
-	SLsmg_write_string ("section from the body by a BLANK line.");
+	SLsmg_write_string ("Your message is not acceptable for the following reason:");
+	slrn_set_color (ERROR_COLOR);
+	SLsmg_gotorc (4,4); SLsmg_write_string (err);
+	if (num && (the_line != NULL))
+	  {
+	     SLsmg_gotorc (6,0);
+	     slrn_set_color (SUBJECT_COLOR);
+	     SLsmg_printf ("This message was generated while looking at line %d:", num);
+	     SLsmg_gotorc (8,0);
+	     slrn_set_color (QUOTE_COLOR);
+	     SLsmg_write_string (the_line);
+	     SLsmg_gotorc (12, 0);
+	     slrn_set_color (ARTICLE_COLOR);
+	     SLsmg_write_string ("Perhaps this error was generated because you did not separate the header");
+	     SLsmg_gotorc (13, 0);
+	     SLsmg_write_string ("section from the body by a BLANK line.");
+	  }
+	return 1;
      }
-   
-   if (only_warning) return 2;
-   return 1;
+   else
+     {
+	int row = 4;
+	SLsmg_write_string ("Your message breaks the following netiquette guidelines:");
+	slrn_set_color (ERROR_COLOR);
+	
+	if (Slrn_Netiquette_Warnings)
+	  {
+	     warn = 0;
+	     while ((warn < MAX_WARNINGS) && warnings[warn] != NULL)
+	       {
+		  SLsmg_gotorc (row++, 4);
+		  SLsmg_write_string (warnings[warn++]);
+	       }
+	  }
+	
+	if ((the_line != NULL) && Slrn_Reject_Long_Lines)
+	  {
+	     SLsmg_gotorc (++row, 4);
+	     SLsmg_write_string ("Lines with more than 80 characters generally need to be wrapped.");
+	     slrn_set_color (SUBJECT_COLOR);
+	     row += 2; SLsmg_gotorc (row, 0);
+	     SLsmg_printf ("This affects at least line %d:", num);
+	     row += 2; SLsmg_gotorc (row, 0);
+	     slrn_set_color (QUOTE_COLOR);
+	     SLsmg_write_string (the_line);
+	     if (Slrn_Reject_Long_Lines == 1)
+	       return 1;
+	  }
+	return 2;
+     }
+   return 0; /* never reached */
 }
 
 
@@ -583,7 +664,7 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
      }
 
 #if SLRN_HAS_MIME
-   if (Slrn_Use_Mime)
+   if (Slrn_Use_Mime & MIME_ARCHIVE)
      slrn_mime_scan_file (infp);
 #endif
    
@@ -621,23 +702,18 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 		    fprintf (outfp, "Message-Id: <slrn%s@%s>\n", msgid,
 			     Slrn_User_Info.posting_host);
 #if SLRN_HAS_MIME
-		  if (Slrn_Use_Mime)
+		  if (Slrn_Use_Mime & MIME_ARCHIVE)
 		    slrn_mime_add_headers (outfp);
 #endif
 		  system_os_name = slrn_get_os_name ();
-#if SLRN_HAS_USER_AGENT
 		  fprintf (outfp, "User-Agent: slrn/%s (%s)\n", Slrn_Version,
 			   system_os_name);
-#else
-		  fprintf (outfp, "X-Newsreader: slrn (%s %s)\n", Slrn_Version,
-			   system_os_name);
-#endif
 		  header = 0;
 	       }
 	     else if (is_empty_header (line))
 	       continue;
 #if SLRN_HAS_MIME
-	     else if (Slrn_Use_Mime)
+	     else if (Slrn_Use_Mime & MIME_ARCHIVE)
 	       slrn_mime_header_encode (line, sizeof(line));
 #endif
 	  }
@@ -656,65 +732,63 @@ int slrn_save_file_to_mail_file (char *file, char *save_file, char *msgid)
 
 static int post_references_header (char *line)
 {
-#define GNKSA_LENGTH 998
-   char buf[GNKSA_LENGTH + 1];
-   char *p;
-   unsigned int total_len, len;
+#define GNKSA_LENGTH 986 /* 998 - strlen ("References: ") */
+   char buf[GNKSA_LENGTH + 2];
+   char *p, *l, *r, *nextid;
+   unsigned int len = 0;
 
    /* Make sure line does not end in whitespace */
    (void) slrn_trim_string (line);
    
-   len = strlen (line);
-   if (len <= GNKSA_LENGTH)
-     {
-	Slrn_Post_Obj->po_puts (line);
-	Slrn_Post_Obj->po_puts ("\n");
-	return 0;
-     }
-
-   /* The reference header is too long so start truncating */
-   /* We have to preserve the first message-id */
-   p = slrn_strchr (line, '>');
-   if (p == NULL) return -1;
-   p++;
-   len = p - line;
-   if (len >= GNKSA_LENGTH)
-     {
-	/* Must have been generated by Pine */
-	Slrn_Post_Obj->po_puts (line);
-	Slrn_Post_Obj->po_puts ("\n");
-	return 0;
-     }
-   strncpy (buf, line, len);
-   buf[len] = 0;
-   Slrn_Post_Obj->po_puts (buf);
-   Slrn_Post_Obj->po_puts (" ");
-   total_len = len + 1;
+   if ((NULL == (l = slrn_strchr (line, '<'))) ||
+       (NULL == (r = slrn_strchr (l+1, '>'))))
+     return -1;
+   while ((NULL != (nextid = slrn_strchr (l+1, '<'))) &&
+	  (nextid < r))
+     l = nextid;
    
-   line += len;
-   if ((*line == '\t') || (*line == ' ')) 
-     line++;	       /* already handled space */
-
-   /* Now use message-ids at end of the line */
-   p = line + strlen (line);
-
-   while (1)
+   len = r - l + 1;
+   if (nextid != NULL) /* Skip enough IDs to fit into our limit */
      {
-	if (*p == '<')
+	nextid--;
+	while (NULL != (nextid = slrn_strchr (nextid + 1, '<')))
 	  {
-	     if (strlen (p) + total_len > GNKSA_LENGTH)
-	       {
-		  p = slrn_strchr (p + 1, '<');
-		  if (p == NULL) p = line;
-		  break;
-	       }
+	     if (strlen (nextid) + len < GNKSA_LENGTH)
+	       break;
 	  }
-	if (p == line) break;
-	p--;
      }
    
-   Slrn_Post_Obj->po_puts (p);
-   Slrn_Post_Obj->po_puts ("\n");
+   /* I'd rather violate GNKSA's limit than omitting the first or last ID */
+   if ((nextid == NULL) || (len >= GNKSA_LENGTH))
+     {
+	Slrn_Post_Obj->po_puts ("References: ");
+	Slrn_Post_Obj->po_puts (l);
+	Slrn_Post_Obj->po_puts ("\n");
+	return 0;
+     }
+   strncpy (buf, l, len);
+   p = buf + len;
+   l = nextid;
+   
+   /* Copy the rest, dropping chopped IDs */
+   while (l != NULL)
+     {
+	if (NULL == (r = slrn_strchr (l+1, '>')))
+	  break;
+	while ((NULL != (nextid = slrn_strchr (l+1, '<'))) &&
+	       (nextid < r))
+	  l = nextid;
+	len = r - l + 1;
+	*p = ' ';
+	strncpy (p+1, l, len);
+	p += len + 1;
+	l = nextid;
+     }
+   
+   strcpy (p, "\n");
+   
+   Slrn_Post_Obj->po_puts ("References: ");
+   Slrn_Post_Obj->po_puts (buf);
    return 0;
 }
 
@@ -822,7 +896,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	       return 1;
 
 	     /* Instead of returning, let user have another go at it */
-	     /* return -1; */
+	     sleep (1);
 	     break;
 	     
 	   case 'd':
@@ -895,10 +969,14 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    msgid = slrn_create_message_id ();
 #endif
 
-   slrn_chmap_fix_file (file, 0);
+   if (Slrn_Use_Mime & MIME_ARCHIVE) /* else: do it later */
+     slrn_chmap_fix_file (file, 0);
    
    if (-1 == slrn_save_file_to_mail_file (file, Slrn_Save_Posts_File, msgid))
      return -1;
+   
+   if ((Slrn_Use_Mime & MIME_ARCHIVE) == 0)
+     slrn_chmap_fix_file (file, 0);
 
    if ((fp = fopen (file, "r")) == NULL)
      {
@@ -907,7 +985,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
      }
    
 #if SLRN_HAS_MIME
-   if (Slrn_Use_Mime)
+   if (Slrn_Use_Mime & MIME_DISPLAY)
      slrn_mime_scan_file (fp);
 #endif
 #if SLRN_HAS_SLANG
@@ -918,6 +996,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	if (SLang_Error)
 	  {
 	     slrn_error ("post_file_hook returned error.  %s not posted.", file);
+	     slrn_chmap_fix_file (file, 1);
 	     (void) saved_failed_post (file);
 	     return -1;
 	  }
@@ -936,21 +1015,24 @@ int slrn_post_file (char *file, char *to, int is_postponed)
    if (status != CONT_POST)
      {
 	fclose (fp);
+	slrn_chmap_fix_file (file, 1);
 	(void) saved_failed_post (file);
 	if (status != -1)
 	  slrn_error ("Posting not allowed.");
 	return -1;
      }
-   
+
+#if 0 /* We shouldn't have to set this one */
    if (Slrn_User_Info.username != NULL)
      Slrn_Post_Obj->po_printf ("Path: %s\n", Slrn_User_Info.username);
+#endif
 
 #if SLRN_HAS_STRICT_FROM
    /* line is not used until now, so we can use it as a buffer */
    slrn_strncpy(line, slrn_make_from_string (), sizeof(line));
 
 # if SLRN_HAS_MIME
-   if (Slrn_Use_Mime)
+   if (Slrn_Use_Mime & MIME_DISPLAY)
      slrn_mime_header_encode(line, sizeof(line));
 # endif
 
@@ -1001,20 +1083,14 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 		    }
 #endif
 #if SLRN_HAS_MIME
-		  if (Slrn_Use_Mime)
+		  if (Slrn_Use_Mime & MIME_DISPLAY)
 		    slrn_mime_add_headers (0);   /* 0 --> Slrn_Post_Obj->po_puts */
 #endif
-#if SLRN_HAS_USER_AGENT
 		  Slrn_Post_Obj->po_printf ("User-Agent: slrn/%s (%s)\n\n", 
-					    Slrn_Version, 
-					    system_os_name);
-#else
-		  Slrn_Post_Obj->po_printf ("X-Newsreader: slrn (%s %s)\n\n", Slrn_Version,
-			       system_os_name);
-#endif
+					    Slrn_Version, system_os_name);
 		  header = 0;
 #if SLRN_HAS_MIME
-		  if (Slrn_Use_Mime) fp = slrn_mime_encode (fp);
+		  if (Slrn_Use_Mime & MIME_DISPLAY) fp = slrn_mime_encode (fp);
 #endif
 		  
 		  continue;
@@ -1037,7 +1113,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 #endif	     
 	     linep[len - 1] = 0;
 #if SLRN_HAS_MIME
-	     if (Slrn_Use_Mime)
+	     if (Slrn_Use_Mime & MIME_DISPLAY)
 	       slrn_mime_header_encode (linep, sizeof (line) - 1);
 #endif
 	  }
@@ -1068,6 +1144,8 @@ int slrn_post_file (char *file, char *to, int is_postponed)
      slrn_message ("Posting...done.");
    else
      {
+	/* convert back to users' charset, so it can easily be re-edited */
+	slrn_chmap_fix_file (file, 1); 
 	if (Slrn_Batch) 
 	  {
 	     saved_failed_post (file);
@@ -1083,8 +1161,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 	if ((rsp == 7)
 	    || (rsp == 'c') || (rsp == 'C')
 	    || (((rsp == 'e') || (rsp == 'E'))
-		&& ((slrn_chmap_fix_file (file, 1) < 0)
-		    || (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0))))
+		&& (slrn_edit_file (Slrn_Editor_Post, file, 1, 0) < 0)))
 	  {
 	     (void) saved_failed_post (file);
 	     return -1;
@@ -1100,9 +1177,7 @@ int slrn_post_file (char *file, char *to, int is_postponed)
 }
 
 
-
-
-int slrn_post (char *newsgroup, char *subj, char *dist)
+int slrn_post (char *newsgroup, char *followupto, char *subj)
 {
    FILE *fp;
    char file[SLRN_MAX_PATH_LEN];
@@ -1133,11 +1208,10 @@ int slrn_post (char *newsgroup, char *subj, char *dist)
 	fprintf (fp, "Organization: %s\n", Slrn_User_Info.org);
      }
    
-   fprintf (fp, "Reply-To: %s\nFollowup-To: \n", Slrn_User_Info.replyto);
+   fprintf (fp, "Reply-To: %s\nFollowup-To: %s\n", Slrn_User_Info.replyto,
+	    followupto);
    fprintf (fp, "Keywords: \nSummary: \n");
    
-   if (dist != NULL) fprintf (fp, "Distribution: %s\n", dist);
-   
    header_lines += slrn_add_custom_headers (fp, Slrn_Post_Custom_Headers, NULL);
 
    fputs ("\n", fp);
