@@ -173,28 +173,6 @@ static int inews_select_post_object (void)
    return 0;
 }
 
-static void inews_usage (void)
-{
-   fputs ("--inews options:\n\
-",
-	  stdout);
-   exit (0);
-}
-
-static int inews_parse_args (char **argv, int argc)
-{
-   int i;
-   
-   for (i = 0; i < argc; i++)
-     {
-	if (!strcmp (argv[i], "--help"))
-	  inews_usage ();
-	else break;
-     }
-   
-   return i;
-}
-
 #endif				       /* HAS_INEWS_SUPPORT */
 
 
@@ -353,9 +331,8 @@ int slrn_parse_object_args (char *name, char **argv, int argc)
    if (!strcmp (name, "spool"))
      {
 #if SLRN_HAS_SPOOL_SUPPORT
-	num_parsed = spool_parse_args (argv, argc);
 	Slrn_Server_Id = SLRN_SERVER_ID_SPOOL;
-	return num_parsed;
+	return 0;
 #else
 	return -2;
 #endif
@@ -364,9 +341,8 @@ int slrn_parse_object_args (char *name, char **argv, int argc)
    if (!strcmp (name, "inews"))
      {
 #if SLRN_HAS_INEWS_SUPPORT
-	num_parsed = inews_parse_args (argv, argc);
 	Slrn_Post_Id = SLRN_POST_ID_INEWS;
-	return num_parsed;
+	return 0;
 #else 
 	return -2;
 #endif
