@@ -47,11 +47,11 @@ typedef struct
    int (*sv_xpat_cmd) (char *, int, int, char *);
 
    int (*sv_xhdr_command) (char *, int, char *, unsigned int);
-   int (*sv_xgtitle_cmd) (char *);
 
    int (*sv_has_cmd) (char *);
+   int (*sv_list) (char *);
    int (*sv_list_newsgroups) (void);
-   int (*sv_list_active) (void);
+   int (*sv_list_active) (char *);
    int (*sv_send_authinfo) (void);
 
    int sv_has_xhdr;
