@@ -81,14 +81,14 @@
 #include "util.h"
 #include "snprintf.h"
 #include "chmap.h"
+#include "post.h"
+#include "xover.h"
+#include "mime.h"
 
 #if SLRN_USE_SLTCP
 # include "sltcp.h"
 #endif
 
-#if SLRN_HAS_MIME 
-# include "mime.h"
-#endif
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
@@ -125,6 +125,8 @@ Slrn_Mode_Type *Slrn_Current_Mode;
 int Slrn_Default_Server_Obj = SLRN_DEFAULT_SERVER_OBJ;
 int Slrn_Default_Post_Obj = SLRN_DEFAULT_POST_OBJ;
 
+FILE *Slrn_Debug_Fp = NULL;
+
 /*}}}*/
 /*{{{ Static Variables */
 
@@ -205,8 +207,7 @@ static int make_lock( char *file ) /*{{{*/
 
 static void lock_file (int how) /*{{{*/
 {
-   char file[SLRN_MAX_PATH_LEN];
-   char name[256];
+   char name[SLRN_MAX_PATH_LEN];
    static int not_ok_to_unlock;
 #if SLRN_HAS_RNLOCK
    int rnlock = 0;
@@ -223,11 +224,10 @@ static void lock_file (int how) /*{{{*/
 #else
    slrn_snprintf (name, sizeof (name), "%s-lock", Slrn_Newsrc_File);
 #endif
-
-   slrn_make_home_filename (name, file, sizeof (file));
    
 #if SLRN_HAS_RNLOCK
-   if (0 == strcmp(".newsrc", Slrn_Newsrc_File))
+   slrn_make_home_filename (".newsrc", file_rn, sizeof (file_rn));
+   if (0 == strcmp (file_rn, Slrn_Newsrc_File))
      {
 	rnlock = 1;
 	slrn_make_home_filename (".rnlock", file_rn, sizeof (file_rn));
@@ -236,26 +236,26 @@ static void lock_file (int how) /*{{{*/
 
    if (how == 1)
      {
-	test_lock (file);
+	test_lock (name);
 #if SLRN_HAS_RNLOCK
 	if (rnlock) test_lock (file_rn);
 #endif
-	if (-1 == make_lock (file))
+	if (-1 == make_lock (name))
 	  {
-	     slrn_exit_error ("Unable to create lock file %s.", file);
+	     slrn_exit_error ("Unable to create lock file %s.", name);
 	  }
 	
 #if SLRN_HAS_RNLOCK
 	if (rnlock && (-1 == make_lock (file_rn)))
 	  {
-	     slrn_delete_file (file); /* delete the "normal" lock file */
+	     slrn_delete_file (name); /* delete the "normal" lock file */
 	     slrn_exit_error ("Unable to create lock file %s.", file_rn);
 	  }
 #endif       
      }
    else
      {
-	if (-1 == slrn_delete_file (file))
+	if (-1 == slrn_delete_file (name))
 	  {
 	     /* slrn_exit_error ("Unable to remove lockfile %s.", file); */
 	  }
@@ -785,18 +785,20 @@ void slrn_enable_mouse (int mode) /*{{{*/
 
 static void perform_cleanup (void)
 {
-   if (Slrn_Server_Obj != NULL) 
-     Slrn_Server_Obj->sv_close ();
-   
    slrn_set_display_state (0);
    
-   lock_file (0);
 #if SLRN_HAS_GROUPLENS
    slrn_close_grouplens ();
 #endif
       
    if (Slrn_Server_Obj != NULL)
      Slrn_Server_Obj->sv_close ();
+
+   if (Slrn_Debug_Fp != NULL)
+     {
+	(void) fclose (Slrn_Debug_Fp);
+	Slrn_Debug_Fp = NULL;
+     }
    
 #if SLRN_HAS_GROUPLENS
    slrn_close_grouplens ();
@@ -856,37 +858,34 @@ void slrn_exit_error (char *fmt, ...) /*{{{*/
 
 static void usage (char *extra) /*{{{*/
 {
-   fputs ("\
-Usage: slrn [--inews ...] [--nntp ...] [--spool ...] OPTIONS\n\
--n              Do not check for new groups.  This usually results in\n\
-                 a faster startup.\n\
+   puts ("\
+Usage: slrn [--inews] [--nntp ...] [--spool] OPTIONS\n\
+-a              Use active file for getting new news.\n\
 -f newsrc-file  Name of the newsrc file to use.\n\
--C              Use colors.\n\
---create        Create a newsrc file by getting list of groups from server.\n\
+-C[-]           [Do not] use colors.\n\
+-Dname          Add 'name' to list of predefined preprocessing tokens.\n\
 -d              Get new text descriptions of each group from server.\n\
                  Note: This may take a LONG time to retrieve this information.\n\
                  The resulting file can be several hundred Kilobytes!\n\
--i init-file    Name of initialization file to use (default .slrnrc)\n\
--Dname          Add 'name' to list of predefined preprocessing tokens.\n\
+-i init-file    Name of initialization file to use (default: "SLRN_USER_SLRNRC_FILENAME")\n\
 -k              Do not process score file.\n\
 -k0             Process score file but inhibit expensive scores.\n\
---kill-log FILE Keep a log of all killed articles in FILE\n\
--a              Use active file for getting new news\n\
 -m              Force XTerm mouse reporting\n\
+-n              Do not check for new groups.  This usually results in\n\
+                 a faster startup.\n\
+-w              Wait for key before switching to full screen mode\n\
+--create        Create a newsrc file by getting list of groups from server.\n\
+--debug FILE    Write debugging information to FILE\n\
 --help          Print this usage.\n\
+--kill-log FILE Keep a log of all killed articles in FILE\n\
 --version       Show version and supported features\n\
 \n\
-For additional info use one of the forms:\n\
-    slrn --inews --help\n\
-    slrn --nntp --help\n\
-    slrn --spool --help\n\
-", 
-	  stderr);
-
+NNTP mode has additional options; use \"slrn --nntp --help\" to display them.\n\
+");
    
    if (extra != NULL)
      {
-	fprintf (stderr, "\n%s\n", extra);
+	printf ("\n%s\n", extra);
      }
    exit (1);
 }
@@ -954,9 +953,10 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    int use_mouse = 0;
    int dsc_flag = 0;
    int use_active = 0;
+   int wait_for_key = 0;
    FILE *fp;
    char file [SLRN_MAX_PATH_LEN];
-   char *init_file = SLRN_USER_SLRNRC_FILENAME;
+   char *init_file = NULL;
    char *kill_logfile = NULL;
 
 #ifdef HAVE_SETLOCALE
@@ -1023,6 +1023,16 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	     else if ((!strcmp ("kill-log", argv_i)) &&
 		      (i + 1 < (unsigned int) argc))
 	       kill_logfile = argv[++i];
+             else if ((!strcmp ("debug", argv_i)) &&
+                      (i + 1 < (unsigned int) argc))
+               {
+		  if (Slrn_Debug_Fp != NULL)
+		    fclose (Slrn_Debug_Fp);
+                  Slrn_Debug_Fp = fopen (argv[++i], "w");
+                  if (Slrn_Debug_Fp == NULL)
+                    slrn_exit_error ("Unable to open %s for debugging.", argv[i]);
+                  setbuf (Slrn_Debug_Fp, (char *) NULL);
+               }
 	     else usage (NULL);
 	  }
 	else if (!strcmp ("-create", argv_i)) create_flag = 1;
@@ -1035,6 +1045,8 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	else if (!strcmp ("-k", argv_i)) no_score_file = 1;
 	else if (!strcmp ("-k0", argv_i))
 	  Slrn_Perform_Scoring &= ~SLRN_EXPENSIVE_SCORING;
+	else if (!strcmp ("-w", argv_i))
+	  wait_for_key = 1;
 	else if (!strncmp ("-D", argv_i, 2) && (argv_i[2] != 0))
 	  {
 	     if (-1 == SLdefine_for_ifdef (argv_i + 2))
@@ -1060,12 +1072,7 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
 	  }
      }
    
-
-#if defined(__DATE__) && defined(__TIME__)
-   fprintf (stdout, "slrn %s (%s %s)\n", Slrn_Version, __DATE__, __TIME__);
-#else
-   fprintf (stdout, "slrn %s\n", Slrn_Version);
-#endif
+   fprintf (stdout, "slrn %s [%s]\n", Slrn_Version, Slrn_Date);
 
    if (dsc_flag && create_flag)
      {
@@ -1121,7 +1128,23 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
     * SLtt_get_terminfo call above fixed that.
     */
    slrn_read_startup_file (file);      /* global file for all users */
+   
+   if (init_file == NULL)
+     {
+	slrn_make_home_filename (SLRN_USER_SLRNRC_FILENAME, file, sizeof (file));
+	init_file = file;
+     }
+   
    slrn_read_startup_file (init_file);
+   
+   if (Slrn_Saw_Obsolete)
+     {
+	slrn_message ("\n! Your configuration file contains obsolete commands or function names that\n"
+		      "! will not be supported by future versions of this program.\n"
+		      "! If you have Perl installed, you can use the script slrnrc-conv to change\n"
+		      "! your configuration accordingly.  It can be found in the source distribution\n"
+		      "! or retrieved from <URL:ftp://slrn.sourceforge.net/pub/slrn/>.\n");
+     }
 
    if (Slrn_Server_Id == 0) Slrn_Server_Id = Slrn_Default_Server_Obj;
    if (Slrn_Post_Id == 0) Slrn_Post_Id = Slrn_Default_Post_Obj;
@@ -1139,11 +1162,16 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
    if (NULL == Slrn_Mime_Display_Charset)
      {
 # if SLRN_HAS_CHARACTER_MAP
-	if ((NULL != Slrn_Charset) &&
-	    (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
-				    (unsigned char *)"ibm852")))
-	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-2");
-	else
+	if (NULL != Slrn_Charset)
+	  {
+	     if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
+					(unsigned char *)"ibm852"))
+	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-2");
+	     else if (0 == slrn_case_strcmp ((unsigned char *)Slrn_Charset,
+					     (unsigned char *)"koi8"))
+	       Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("koi8-r");
+	  }
+	if (NULL == Slrn_Mime_Display_Charset)
 # endif
 	  Slrn_Mime_Display_Charset = slrn_safe_strmalloc ("iso-8859-1");
      }
@@ -1174,8 +1202,8 @@ static int main_init_and_parse_args (int argc, char **argv) /*{{{*/
        || (0 == slrn_is_fqdn (Slrn_User_Info.hostname)))
      {
 	slrn_exit_error ("\
-Unable to find a fully qualified hostname.  You will have to specify a\r\n\
-hostname in your %s file.\r\n", 
+Unable to find a valid hostname for constructing your e-mail address.\r\n\
+You will have to specify a hostname in your %s file.\r\n", 
 			 SLRN_USER_SLRNRC_FILENAME);
      }
    if ((NULL == Slrn_User_Info.username)
@@ -1187,10 +1215,13 @@ Unable to find your user name.  This means that a valid 'From' header line\r\n\
 cannot be constructed.  Try setting the USER environment variable.\r\n");
      }
 
-   if (NULL == Slrn_User_Info.posting_host)
+   if ((NULL == Slrn_User_Info.posting_host)
+       && Slrn_Generate_Message_Id)
      slrn_message_now ("\
-***Warning: unable to find a fully-qualified host name\r\n\
-            slrn will not generate any Message-IDs.\r\n");
+***Warning: Unable to find a unique fully-qualified host name.\r\n\
+            slrn will not generate any Message-IDs.\r\n\
+            Please note that the \"hostname\" setting does not affect this;\r\n\
+            see the \"slrn reference manual\" for details.\r\n");
    
    if (no_score_file == 0) read_score_file ();
    
@@ -1202,11 +1233,16 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
    
    if ((Slrn_Newsrc_File == NULL)
        && ((Slrn_Newsrc_File = slrn_map_file_to_host (Slrn_Server_Obj->sv_name)) == NULL))
+     {
 #if defined(VMS) || defined(IBMPC_SYSTEM)
-     Slrn_Newsrc_File = "jnews.rc";
+	Slrn_Newsrc_File = "jnews.rc";
 #else
-   Slrn_Newsrc_File = ".jnewsrc";
+	Slrn_Newsrc_File = ".jnewsrc";
 #endif
+	slrn_make_home_filename (Slrn_Newsrc_File, file, sizeof (file));
+	Slrn_Newsrc_File = slrn_safe_strmalloc (file);
+     }
+   
    slrn_message_now ("Using newsrc file %s for server %s.",
 		     Slrn_Newsrc_File, Slrn_Server_Obj->sv_name);
    
@@ -1228,13 +1264,12 @@ cannot be constructed.  Try setting the USER environment variable.\r\n");
    
    if (create_flag == 0)
      {
-	/* Check to see if the .newrc file exists--- I should use the access
+	/* Check to see if the .newsrc file exists -- I should use the access
 	 * system call but for now, do it this way.
 	 */
-	if (NULL == (fp = slrn_open_home_file (Slrn_Newsrc_File, "r", file,
-					       sizeof (file), 0)))
+	if (NULL == (fp = fopen (Slrn_Newsrc_File, "r")))
 	  {
-	     slrn_error ("Unable to open %s.  I will try .newsrc.", file);
+	     slrn_error ("Unable to open %s.  I will try .newsrc.", Slrn_Newsrc_File);
 	     if (NULL == (fp = slrn_open_home_file (".newsrc", "r", file,
 						    sizeof (file), 0)))
 	       {
@@ -1280,6 +1315,12 @@ If you want to create %s, add command line options:\r\n\
 	slrn_exit_error ("Failed to initialize server.");
      }
    
+   if (Slrn_Server_Obj->sv_has_xover == 1)
+     {
+	if (0 == (Slrn_Server_Obj->sv_has_xover = slrn_read_overview_fmt ()))
+	  slrn_message ("Broken OVERVIEW.FMT -- XOVER support disabled.");
+     }
+   
    if (Slrn_Check_New_Groups || create_flag)
      {
 	slrn_get_new_groups (create_flag);
@@ -1288,14 +1329,22 @@ If you want to create %s, add command line options:\r\n\
    slrn_read_newsrc (create_flag);
    slrn_read_group_descriptions ();
    
-   putc ('\n', stdout);
+   if (wait_for_key)
+     {
+	slrn_message ("* Press Ctrl-C to quit, any other key to continue.\n");
+	slrn_set_display_state (SLRN_TTY_INIT);
+	if ('\003' == SLang_getkey ())
+	  slrn_exit_error ("Exit on user request.");
+     }
+   else
+     putc ('\n', stdout);
    
    slrn_set_display_state (SLRN_SMG_INIT | SLRN_TTY_INIT);
-
+   
 #if defined(__unix__) && !defined(IBMPC_SYSTEM)
    if (Slrn_Autobaud) SLtt_Baud_Rate = SLang_TT_Baud_Rate;
 #endif
-
+   
    return 0;
 }
 
