@@ -83,6 +83,7 @@ extern int Slrn_Default_Post_Obj;
 
 extern char *Slrn_Lib_Dir;
 extern int Slrn_Batch;
+extern FILE *Slrn_Debug_Fp;
 
 extern int slrn_sys_system (char *);
 
@@ -111,19 +112,21 @@ extern int slrn_sys_system (char *);
 #define SELECT_COLOR		23
 #define VERBATIM_COLOR		24
 #define DATE_COLOR		36
-/* SUBJECT_COLOR + 5 == UNREAD_SUBJECT_COLOR etc. */
+/* SUBJECT_COLOR + 5 == HL_SUBJECT_COLOR etc. */
 #define SUBJECT_COLOR		40
 #define NEG_SCORE_COLOR		41
 #define POS_SCORE_COLOR		42
 #define HIGH_SCORE_COLOR	43
-#define UNREAD_SUBJECT_COLOR	45
-#define UNREAD_NEG_SCORE_COLOR	46
-#define UNREAD_POS_SCORE_COLOR	47
-#define UNREAD_HIGH_SCORE_COLOR	48
+#define UNREAD_SUBJECT_COLOR	44
+#define HL_SUBJECT_COLOR	45
+#define HL_NEG_SCORE_COLOR	46
+#define HL_POS_SCORE_COLOR	47
+#define HL_HIGH_SCORE_COLOR	48
 
 #define BOLD_COLOR		30
 #define UNDERLINE_COLOR		31
 #define ITALICS_COLOR		32
+#define URL_COLOR		33
 #define QUOTE_COLOR	50
 
 #endif				       /* _SLRN_SLRN_H_ */
