@@ -75,6 +75,7 @@
 #if SLRN_HAS_MIME
 # include "mime.c"
 #endif
+
 #include "xover.c"
 
 #undef SLRN_HAS_MSGID_CACHE
@@ -88,44 +89,12 @@
 
 static char *Slrnpull_Version = SLRN_VERSION;
 
-#ifndef SLRNPULL_ROOT_DIR
-# define SLRNPULL_ROOT_DIR	"/var/spool/news/slrn"
-#endif
-
-#ifndef SLRNPULL_CONF
-# define SLRNPULL_CONF		"slrnpull.conf"
-#endif
-
-#ifndef SLRNPULL_OUTGOING_DIR
-# define SLRNPULL_OUTGOING_DIR	"out.going"
-#endif
-
 #if SLRNPULL_USE_SETGID_POSTS
 # define OUTGOING_DIR_MODE	(0777 | 03000)
 #else
 # define OUTGOING_DIR_MODE	(0777 | 01000)
 #endif
 
-#ifndef SLRNPULL_SCORE_FILE
-# define SLRNPULL_SCORE_FILE	"score"
-#endif
-
-#ifndef SLRNPULL_NEWS_DIR
-# define SLRNPULL_NEWS_DIR	"news"
-#endif
-
-#ifndef SLRNPULL_LOGFILE
-# define SLRNPULL_LOGFILE	"log"
-#endif
-
-#ifndef SLRNPULL_OUTGOING_BAD_DIR
-# define SLRNPULL_OUTGOING_BAD_DIR	"rejects"
-#endif
-
-#ifndef SLRNPULL_OUTGOING_DIR_MODE
-# define SLRNPULL_OUTGOING_DIR_MODE	(0777 | 01000)
-#endif
-
 static int Exit_Code;
 #define SLRN_EXIT_UNKNOWN		1
 #define SLRN_EXIT_BAD_USAGE		2
@@ -136,6 +105,7 @@ static int Exit_Code;
 
 #define SLRN_EXIT_FILEIO		20
 
+FILE *Slrn_Debug_Fp = NULL;
 char *SlrnPull_Dir = SLRNPULL_ROOT_DIR;
 char *SlrnPull_Spool_News_Dir;
 char *Group_Min_Max_File;	       /* relative to group dir */
@@ -1043,7 +1013,11 @@ static int write_head_and_body (Active_Group_Type *g, int n, /*{{{*/
        || (-1 == slrn_dircat (file, buf, file, sizeof (file))))
      return -1;
    
+#ifdef __OS2__
+   fp = fopen (file, "wb");
+#else
    fp = fopen (file, "w");
+#endif
    if (fp == NULL)
      {
 	log_error ("Unable to open %s for writing.", file);
@@ -1274,10 +1248,12 @@ static FILE *open_xover_file (Active_Group_Type *g, char *mode) /*{{{*/
 static int get_articles (NNTP_Type *s, Active_Group_Type *g, int *numbers, unsigned int num) /*{{{*/
 {
    unsigned int i;
-#define MAX_QUEUED 10
-   char *heads[MAX_QUEUED];
-   char *bodies[MAX_QUEUED];
-   Slrn_XOver_Type xovs [MAX_QUEUED];
+#ifndef SLRN_MAX_QUEUED
+# define SLRN_MAX_QUEUED 10
+#endif
+   char *heads[SLRN_MAX_QUEUED];
+   char *bodies[SLRN_MAX_QUEUED];
+   Slrn_XOver_Type xovs [SLRN_MAX_QUEUED];
    int ret;
    FILE *fp;
    
@@ -1381,11 +1357,11 @@ static int get_group_articles (NNTP_Type *s, Active_Group_Type *g,
    
    while (i < num_numbers)
      {
-	int ns[MAX_QUEUED];
+	int ns[SLRN_MAX_QUEUED];
 	unsigned int j;
 	
 	j = 0;
-	while ((i < num_numbers) && (j < MAX_QUEUED))
+	while ((i < num_numbers) && (j < SLRN_MAX_QUEUED))
 	  {
 	     ns[j] = numbers[i];
 	     i++;
@@ -1691,13 +1667,7 @@ static int open_servers (char *host) /*{{{*/
    time (&Start_Time);
    
    /* Probe for XHDR now because DNEWS does not handle probing later properly. */
-   if (-1 == nntp_has_cmd (Pull_Server, "XHDR"))
-     {
-#if SLRN_USE_SLTCP
-	sltcp_close_sltcp ();
-#endif
-	return -1;
-     }
+   (void) nntp_has_cmd (Pull_Server, "XHDR");
    return 0;
 }
 
@@ -1913,13 +1883,13 @@ int main (int argc, char **argv) /*{{{*/
 	  }
 	else if (!strcmp (arg, "--debug") && (argc > 0))
 	  {
-	     if (NULL == (NNTP_Debug_Fp = fopen (*argv, "w")))
+	     if (NULL == (Slrn_Debug_Fp = fopen (*argv, "w")))
 	       {
 		  fprintf (stderr, "Unable to create debug file %s\n", *argv);
 		  exit (1);
 	       }
              else
-	       setbuf (NNTP_Debug_Fp, (char*) NULL);
+	       setbuf (Slrn_Debug_Fp, (char*) NULL);
 	     
 	     argv++;
 	     argc--;
