@@ -45,9 +45,7 @@
 #include "score.h"
 #include "util.h"
 #include "uudecode.h"
-#if SLRN_HAS_MIME
-# include "mime.h"
-#endif
+#include "mime.h"
 #if SLRN_HAS_GROUPLENS
 # include "grplens.h"
 #endif
@@ -82,6 +80,7 @@ static int grouplens_fun (int, SLcmd_Cmd_Table_Type *);
 static int interpret_fun (int, SLcmd_Cmd_Table_Type *);
 static int include_file_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_header_format_fun (int, SLcmd_Cmd_Table_Type *);
+static int set_group_format_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_visible_headers_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_comp_charsets_fun (int, SLcmd_Cmd_Table_Type *);
 static int set_posting_host (int, SLcmd_Cmd_Table_Type *);
@@ -117,6 +116,7 @@ static SLcmd_Cmd_Type Slrn_Startup_File_Cmds[] = /*{{{*/
      {interpret_fun, "interpret", "S"},
      {include_file_fun, "include", "S"},
      {set_header_format_fun, "header_display_format", "IS"},
+     {set_group_format_fun, "group_display_format", "IS"},
      {set_visible_headers_fun, "visible_headers", "S"},
      {set_comp_charsets_fun, "compatible_charsets", "S"},
      {set_posting_host, "posting_host", "S"},
@@ -151,6 +151,7 @@ SLRegexp_Type *Slrn_Strip_Was_Regexp [SLRN_MAX_REGEXP + 1];
 int Slrn_Autobaud = 0;
 char *Slrn_Score_File;
 int Slrn_Scroll_By_Page;
+int Slrn_Saw_Obsolete = 0;
 
 /*}}}*/
 
@@ -188,6 +189,7 @@ static void exit_unknown_object (void) /*{{{*/
 
 static void issue_obsolete_message (void) /*{{{*/
 {
+   Slrn_Saw_Obsolete = 1;
    slrn_message ("%s: Command is obsolete on line %d:\n%s",
 		 This_File, This_Line_Num, This_Line);
    slrn_message ("The new usage is:\nset %s\n", This_Line);
@@ -199,17 +201,69 @@ static void issue_obsolete_message (void) /*{{{*/
 /*}}}*/
 /*{{{ Set/Unset Key Functions */
 
+static char *Group_Obsolete_Names [] =
+{
+   "down", "line_down",
+     "group_bob", "bob",
+     "group_eob", "eob",
+     "pagedown", "page_down",
+     "pageup", "page_up",
+     "toggle_group_display", "toggle_group_formats",
+     "uncatch_up", "uncatchup",
+     "up", "line_up",
+     NULL
+};
+
+static char *Art_Obsolete_Names [] =
+{
+   "art_bob", "article_bob",
+     "art_eob", "article_eob",
+     "art_xpunge", "article_expunge",
+     "article_linedn", "article_line_down",
+     "article_lineup", "article_line_up",
+     "article_pagedn", "article_page_down",
+     "article_pageup", "article_page_up",
+     "down", "header_line_down",
+     "enlarge_window", "enlarge_article_window",
+     "goto_beginning", "article_bob",
+     "goto_end", "article_eob",
+     "left", "article_left",
+     "locate_header_by_msgid", "locate_article",
+     "pagedn", "header_page_down",
+     "pageup", "header_page_up",
+     "pipe_article", "pipe",
+     "prev", "previous",
+     "print_article", "print",
+     "right", "article_right",
+     "scroll_dn", "article_page_down",
+     "scroll_up", "article_page_up",
+     "shrink_window", "shrink_article_window",
+     "skip_to_prev_group", "skip_to_previous_group",
+     "toggle_show_author", "toggle_header_formats",
+     "up", "header_line_up",
+     NULL
+};
+
 static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
 {
    char *map = table->string_args[1];
    char *fun = table->string_args[2];
    char *key = table->string_args[3];
    SLKeyMap_List_Type *kmap = NULL;
+   char **obsolete_names = NULL;
    
    (void) argc;
 
-   if (!strcmp (map, "group")) kmap = Slrn_Group_Keymap;
-   else if (!strcmp (map, "article")) kmap = Slrn_Article_Keymap;
+   if (!strcmp (map, "group"))
+     {
+	kmap = Slrn_Group_Keymap;
+	obsolete_names = Group_Obsolete_Names;
+     }
+   else if (!strcmp (map, "article"))
+     {
+	kmap = Slrn_Article_Keymap;
+	obsolete_names = Art_Obsolete_Names;
+     }
    else if (!strcmp (map, "readline")) kmap = Slrn_RLine_Keymap;
    else slrn_exit_error ("%s: line %d:\n%sNo such keymap: %s", This_File, This_Line_Num, This_Line, map);
    
@@ -217,6 +271,28 @@ static int setkey_fun (int argc, SLcmd_Cmd_Table_Type *table) /*{{{*/
      {
 	slrn_exit_error ("%s: line %d:\n%serror defining key.", This_File, This_Line_Num, This_Line);
      }
+   else if (obsolete_names != NULL)
+     {
+	int i = 0;
+	char *old_name = NULL, *new_name = NULL;
+	
+	while ((old_name = obsolete_names[i]) != NULL)
+	  {
+	     if (!strcmp (old_name, fun))
+	       {
+		  Slrn_Saw_Obsolete = 1;
+		  new_name = obsolete_names[i+1];
+		  break;
+	       }
+	     i += 2;
+	  }
+	
+	if (new_name != NULL)
+	  slrn_message ("%s: Obsolete function name on line %d: %s\n"
+			"The new name is: %s",
+			This_File, This_Line_Num, old_name, new_name);
+     }
+
    return 0;
 }
 
@@ -356,6 +432,12 @@ static int set_header_format_fun (int argc, SLcmd_Cmd_Table_Type *table)
    return slrn_set_header_format (table->int_args[1], table->string_args[2]);
 }
 
+static int set_group_format_fun (int argc, SLcmd_Cmd_Table_Type *table) 
+{
+   (void) argc;
+   return slrn_set_group_format (table->int_args[1], table->string_args[2]);
+}
+
 static int set_comp_charsets_fun (int argc, SLcmd_Cmd_Table_Type *table)
 {
    (void) argc;
@@ -385,7 +467,6 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"emphasized_text_mask",	&Slrn_Emphasized_Text_Mask},
      {"emphasized_text_mode",	&Slrn_Emphasized_Text_Mode},
      {"process_verbatim_marks", &Slrn_Process_Verbatim_Marks},
-     {"process_verbatum_marks", &Slrn_Process_Verbatim_Marks}, /* FIXME */
 
      {"use_flow_control",	&Slrn_Use_Flow_Control},
      {"abort_unmodified_edits", &Slrn_Abort_Unmodified},
@@ -399,6 +480,7 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"use_color", &SLtt_Use_Ansi_Colors},
      {"ignore_signature", &Slrn_Sig_Is_End_Of_Article},
      {"reject_long_lines", &Slrn_Reject_Long_Lines},
+     {"netiquette_warnings", &Slrn_Netiquette_Warnings},
      {"generate_message_id", &Slrn_Generate_Message_Id},
 #if SLRN_HAS_STRICT_FROM
      {"generate_email_from", NULL},
@@ -413,9 +495,8 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #endif
      {"color_by_score", &Slrn_Color_By_Score},
      {"highlight_unread_subjects", &Slrn_Highlight_Unread},
-     {"use_xgtitle", &Slrn_Use_Xgtitle},
+     {"highlight_urls", &Slrn_Highlight_Urls},
      {"show_article", &Slrn_Startup_With_Article},
-     {"show_descriptions", &Slrn_Group_Display_Descriptions},
      {"smart_quote", &Slrn_Smart_Quote},
      {"no_backups", &Slrn_No_Backups},
      {"beep", &SLtt_Ignore_Beep},
@@ -431,11 +512,11 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
      {"sorting_method", &Slrn_Sorting_Mode},
      {"uncollapse_threads", &Slrn_Threads_Visible},
      {"read_active", &Slrn_List_Active_File},
+     {"drop_bogus_groups", &Slrn_Drop_Bogus_Groups},
      {"prefer_head", &Slrn_Prefer_Head},
 #if SLRN_HAS_MIME
      {"use_metamail", &Slrn_Use_Meta_Mail},
 #endif
-     {"group_dsc_start_column", &Slrn_Group_Description_Column},
      {"lines_per_update", &Slrn_Reads_Per_Update},
      {"min_high_score", &Slrn_High_Score_Min},
      {"max_low_score", &Slrn_Low_Score_Max},
@@ -490,9 +571,13 @@ static Set_Int_Type Int_Things_To_Set [] = /*{{{*/
 #if 1 /* FIXME: These will be removed before 1.0 */
      {"author_display", NULL},
      {"display_author_realname", NULL},
+     {"display_score", NULL},
+     {"group_dsc_start_column", NULL},
+     {"process_verbatum_marks", &Slrn_Process_Verbatim_Marks},
      {"prompt_next_group", NULL},
      {"query_reconnect", NULL},
-     {"display_score", &Slrn_Display_Score},
+     {"show_descriptions", NULL},
+     {"use_xgtitle", NULL},
 #endif
      {NULL, NULL}
 };
@@ -521,12 +606,14 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"username", NULL},
 #endif
      {"art_help_line", &Slrn_Art_Help_Line},
+     {"art_status_line", &Slrn_Art_Status_Line},
      {"header_help_line", &Slrn_Header_Help_Line},
+     {"header_status_line", &Slrn_Header_Status_Line},
      {"group_help_line", &Slrn_Group_Help_Line},
      {"quote_string", &Slrn_Quote_String},
      {"replyto", &Slrn_User_Info.replyto},
      {"organization", &Slrn_User_Info.org},
-     {"followup", &Slrn_User_Info.followup_string},   /* obsolete */
+     {"followup", &Slrn_User_Info.followup_string}, /* FIXME: obsolete */
      {"followup_string", &Slrn_User_Info.followup_string},
      {"reply_string", &Slrn_User_Info.reply_string},
      {"cc_followup_string", &Slrn_CC_Followup_Message},
@@ -647,6 +734,7 @@ static Set_String_Type String_Things_To_Set [] = /*{{{*/
      {"post_object", &Post_Object},
      {"printer_name", &Slrn_Printer_Name},
      {"top_status_line", &Slrn_Top_Status_Line},
+     {"group_status_line", &Slrn_Group_Status_Line},
      {"scorefile", &Slrn_Score_File},
      {NULL, NULL}
 };
@@ -662,7 +750,15 @@ int slrn_set_string_variable (char *name, char *value) /*{{{*/
 	if (!strcmp (sp->what, name))
 	  {
 	     char *ss;
-		  
+	     
+	     if ((This_File != NULL) && !strcmp (name, "followup"))
+	       {
+		  slrn_message ("%s: Obsolete variable name on line %d: followup\n"
+				"The new name is followup_string",
+				This_File, This_Line_Num);
+		  Slrn_Saw_Obsolete = 1;
+	       }
+	     
 	     if (sp->svaluep == NULL) return 0;
 	     
 	     ss = *sp->svaluep;
@@ -689,6 +785,41 @@ int slrn_set_integer_variable (char *name, int value) /*{{{*/
      {
 	if (!strcmp (ip->what, name))
 	  {
+	     if (This_File != NULL) /*{{{ Test for obsolete variables */
+	       {
+		  if (!strcmp (name, "author_display") ||
+		      !strcmp (name, "display_author_realname") ||
+		      !strcmp (name, "display_score"))
+		    {
+		       slrn_message ("%s: Obsolete variable on line %d: %s\n"
+				     "Please make use of the header_display_format command instead.",
+				     This_File, This_Line_Num, name);
+		       Slrn_Saw_Obsolete = 1;
+		    }
+		  else if (!strcmp (name, "group_dsc_start_column") ||
+			   !strcmp (name, "show_descriptions"))
+		    {
+		       slrn_message ("%s: Obsolete variable on line %d: %s\n"
+				     "Please make use of the group_display_format variable instead.",
+				     This_File, This_Line_Num, name);
+		       Slrn_Saw_Obsolete = 1;
+		    }
+		  else if (!strcmp (name, "process_verbatum_marks"))
+		    {
+		       slrn_message ("%s: Obsolete spelling on line %d:\n"
+				     "Use \"process_verbatim_marks\" instead of \"process_verbatum_marks\".",
+				     This_File, This_Line_Num);
+		       Slrn_Saw_Obsolete = 1;
+		    }
+		  else if (!strcmp (name, "prompt_next_group") ||
+			   !strcmp (name, "query_reconnect") ||
+			   !strcmp (name, "use_xgtitle"))
+		    {
+		       slrn_message ("%s: Obsolete variable on line %d: %s",
+				     This_File, This_Line_Num, name);
+		       Slrn_Saw_Obsolete = 1;
+		    }
+	       } /*}}}*/
 	     if (ip->valuep == NULL) return 0;
 	     *ip->valuep = value;
 	     return 0;
@@ -779,6 +910,8 @@ Color_Handle_Type;
 
 static Color_Handle_Type Color_Handles[] = /*{{{*/
 {
+     {"verbatum", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
+   /* FIXME: keep this as the first entry until it gets removed */
      {"article",	ARTICLE_COLOR,		DEF_FG,		DEF_BG, 0},
      {"author",		AUTHOR_COLOR,		"magenta",	DEF_BG, 0},
      {"boldtext", 	BOLD_COLOR,		"yellow", 	DEF_BG, SLTT_BOLD_MASK},
@@ -812,7 +945,8 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
      {"tilde",		SLRN_TILDE_COLOR,	"green",	DEF_BG, SLTT_BOLD_MASK},
      {"tree",		TREE_COLOR,		"red",		DEF_BG, 0},
      {"underlinetext", 	UNDERLINE_COLOR,	"magenta", 	DEF_BG, SLTT_ULINE_MASK},
-     {"verbatum", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0}, /* FIXME */
+     {"unread_subject",	UNREAD_SUBJECT_COLOR,	"white",	DEF_BG, SLTT_BOLD_MASK},
+     {"url", 		URL_COLOR,		"white",	DEF_BG, SLTT_BOLD_MASK},
      {"verbatim", 	VERBATIM_COLOR,		"green", 	DEF_BG, 0},
 
      {NULL, -1, NULL, NULL, 0}
@@ -823,10 +957,10 @@ static Color_Handle_Type Color_Handles[] = /*{{{*/
 /* These are for internal use and automatically set by slrn */
 static Color_Handle_Type Auto_Color_Handles[] = /*{{{*/
 {
-     {"high_score",	UNREAD_HIGH_SCORE_COLOR,	"brightred",	DEF_BG,	SLTT_BOLD_MASK},
-     {"neg_score",	UNREAD_NEG_SCORE_COLOR,	"brightgreen",	DEF_BG,	SLTT_BOLD_MASK},
-     {"pos_score",	UNREAD_POS_SCORE_COLOR,	"brightblue",	DEF_BG, SLTT_REV_MASK | SLTT_BOLD_MASK},
-     {"subject",	UNREAD_SUBJECT_COLOR,	"white",	DEF_BG,	SLTT_BOLD_MASK},
+     {"high_score",	HL_HIGH_SCORE_COLOR,	"brightred",	DEF_BG,	SLTT_BOLD_MASK},
+     {"neg_score",	HL_NEG_SCORE_COLOR,	"brightgreen",	DEF_BG,	SLTT_BOLD_MASK},
+     {"pos_score",	HL_POS_SCORE_COLOR,	"brightblue",	DEF_BG, SLTT_REV_MASK | SLTT_BOLD_MASK},
+     {"subject",	HL_SUBJECT_COLOR,	"white",	DEF_BG,	SLTT_BOLD_MASK},
      {NULL, -1, NULL, NULL, 0}
 };
 
@@ -849,6 +983,14 @@ int slrn_set_object_color (char *name, char *fg, char *bg)
 	  {
 	     Color_Handle_Type *ac = Auto_Color_Handles;
 	     
+	     if ((This_File != NULL) && (ct == Color_Handles))
+	       {
+		  slrn_message ("%s: Obsolete spelling on line %d:\n"
+				"Use \"verbatim\" instead of \"verbatum\"",
+				This_File, This_Line_Num);
+		  Slrn_Saw_Obsolete = 1;
+	       }
+	     
 	     SLtt_set_color (ct->value, name, fg, bg);
 	     
 	     while (ac->name != NULL)
@@ -1141,8 +1283,10 @@ static int add_to_server_list (char *host, char *file, char *username, char *pas
    
    if (file != NULL)
      {
+	char pathbuf [SLRN_MAX_PATH_LEN];
 	slrn_free (s->file);
-	s->file = slrn_safe_strmalloc (file);
+	slrn_make_home_filename (file, pathbuf, sizeof (pathbuf));
+	s->file = slrn_safe_strmalloc (pathbuf);
      }
 
    if (username != NULL)
@@ -1297,7 +1441,7 @@ void slrn_startup_initialize (void) /*{{{*/
    slrn_init_modes ();
    SLang_init_case_tables ();
    
-   Slrn_Ignore_Quote_Regexp[0] = compile_quote_regexp ("^ ?[:>]");
+   Slrn_Ignore_Quote_Regexp[0] = compile_quote_regexp ("^ ? ?[><:=|]");
    Slrn_Strip_Sig_Regexp[0] = compile_quote_regexp ("^-- $");
    
    h = Color_Handles;
@@ -1327,13 +1471,16 @@ void slrn_startup_initialize (void) /*{{{*/
 
    /* The following are required by GNKSA */
    (void) slrn_set_visible_headers ("From:,Subject:,Newsgroups:,Followup-To:,Reply-To:");
-				       
+   
    (void) slrn_set_header_format (0, "%F%-5S%G%-5l:[%12r]%t%s");
    (void) slrn_set_header_format (1, "%F%G%-5l:[%12r]%t%s");
    (void) slrn_set_header_format (2, "%F%-5l:%t%s");
    (void) slrn_set_header_format (3, "%F%-5S%-5l:%t%50s %r");
    (void) slrn_set_header_format (4, "%F%-5S [%10r]:%t%49s %-19g[%17d]");
    
+   (void) slrn_set_group_format (0, "  %F%-5u  %n%45g%d");
+   (void) slrn_set_group_format (1, "  %F%-5u  %n%50g%-8l-%h");
+   (void) slrn_set_group_format (2, "  %F%-5u [%-6t]  %n");
 }
 
 
@@ -1342,7 +1489,6 @@ void slrn_startup_initialize (void) /*{{{*/
 int slrn_read_startup_file (char *name) /*{{{*/
 {
    FILE *fp;
-   char file [SLRN_MAX_PATH_LEN];
    char line [512];
    SLPreprocess_Type pt;
    int save_this_line_num;
@@ -1354,23 +1500,21 @@ int slrn_read_startup_file (char *name) /*{{{*/
 	slrn_exit_error ("Unable to initialize S-Lang readline library.");
      }
    
-
    if (-1 == SLprep_open_prep (&pt))
      {
 	slrn_exit_error ("Error initializing S-Lang preprocessor.");
      }
    
-   
-   fp = slrn_open_home_file (name, "r", file, sizeof (file), 0);
+   fp = fopen (name, "r");
    if (fp == NULL) return -1;
    
-   slrn_message ("Reading startup file %s.", file);
+   slrn_message ("Reading startup file %s.", name);
    
    save_this_file = This_File;
    save_this_line = This_Line;
    save_this_line_num = This_Line_Num;
 
-   This_File = file;
+   This_File = name;
    This_Line = line;
    This_Line_Num = 0;
    
