@@ -42,6 +42,7 @@
 
 #include "util.h"
 #include "ttymsg.h"
+#include "snprintf.h"
 
 /* This function allows NULL as a parameter. This fact _is_ exploited */
 char *slrn_skip_whitespace (char *b) /*{{{*/
@@ -197,12 +198,12 @@ void slrn_os2_convert_path (char *path)
 #endif
 
 #ifdef SLRN_USE_OS2_FAT
-void slrn_os2_make_fat (char *file, char *name, char *ext)
+void slrn_os2_make_fat (char *file, size_t n, char *name, char *ext)
 {
    static char drive[3] = " :";
    char fsys[5];
 
-   strcpy (file, name);
+   slrn_strncpy (file, name, n);
    if (isalpha(file[0]) && (file[1] == ':'))
      drive[0] = file[0];
    else
@@ -215,7 +216,8 @@ void slrn_os2_make_fat (char *file, char *name, char *ext)
 	_remext (file);                      /* Remove the extension */
      }
 
-   strcat (file, ext);
+   if (strlen (file) + strlen (ext) < n)
+     strcat (file, ext); /* safe */
 }
 #endif
 
@@ -240,7 +242,7 @@ static void fixup_path (char *path) /*{{{*/
 /* dir and file could be the same in which case this performs a strcat. 
  * If name looks like an absolute path, it will be returned.
  */
-int slrn_dircat (char *dir, char *name, char *file)
+int slrn_dircat (char *dir, char *name, char *file, size_t n)
 {
    unsigned int len = 0;
    
@@ -248,7 +250,7 @@ int slrn_dircat (char *dir, char *name, char *file)
      {
 	if (slrn_is_absolute_path (name))
 	  {
-	     strcpy (file, name);
+	     slrn_strncpy (file, name, n);
 #if defined(IBMPC_SYSTEM)
 	     slrn_os2_convert_path (file);
 #endif
@@ -261,7 +263,7 @@ int slrn_dircat (char *dir, char *name, char *file)
    if (dir != NULL) len += strlen (dir);
    
    len += 2;			       /* for / and \0 */
-   if (len > SLRN_MAX_PATH_LEN)
+   if (len > n)
      {
 	slrn_error ("File name too long.");
 	return -1;
@@ -269,12 +271,12 @@ int slrn_dircat (char *dir, char *name, char *file)
    
    if (dir != NULL) 
      {
-	if (dir != file) strcpy (file, dir);
+	if (dir != file) strcpy (file, dir); /* safe */
 	fixup_path (file);
      }
    else *file = 0;
 
-   if (name != NULL) strcat (file, name);
+   if (name != NULL) strcat (file, name); /* safe */
 #if defined(IBMPC_SYSTEM)
    slrn_os2_convert_path (file);
 #endif
@@ -436,13 +438,13 @@ char *slrn_spool_dircat (char *root, char *name, int map_dots)
 	slrn_exit_error ("Out of memory.");
      }
 
-   strcpy (spool_group, root);
+   strcpy (spool_group, root); /* safe */
 
    p = spool_group + len;
    if (len && (*(p - 1) != SLRN_PATH_SLASH_CHAR))
      *p++ = SLRN_PATH_SLASH_CHAR;
 
-   strcpy (p, name);
+   strcpy (p, name); /* safe */
 
    if (map_dots) while ((ch = *p) != 0)
      {
