@@ -3,6 +3,7 @@
  This file is part of SLRN.
 
  Copyright (c) 1994, 1999 John E. Davis <davis@space.mit.edu>
+ Copyright (c) 2001 Thomas Schultz <tststs@gmx.de>
 
  This program is free software; you can redistribute it and/or modify it
  under the terms of the GNU General Public License as published by the Free
@@ -41,6 +42,7 @@
 #define MAX_ARTICLE_LINE_LEN 4096
 #include "uudecode.h"
 #include "util.h"
+#include "snprintf.h"
 #include "ttymsg.h"
 #include "slrndir.h"
 
@@ -264,8 +266,28 @@ static int base64_decode_line (char *line, FILE *fp) /*{{{*/
 
 /*}}}*/
 
-	       
-	
+static char *fgets_with_counter (char *s, int size, FILE* stream, /*{{{*/
+				 unsigned int *c)
+{
+   char *retval;
+   int len;
+   
+   retval = fgets (s, size, stream);
+   len = strlen (s);
+   if (len && (s[len - 1] == '\n'))
+     {
+	s[--len] = 0;
+	if (len && (s[len - 1] == '\r'))
+	  s[--len] = 0;
+     }
+   if (c != NULL)
+     *c = len;
+   
+   return retval;
+}
+
+/*}}}*/
+
 static char Base64_Filename[256];
 static int Base64_Unknown_Number;
 
@@ -278,7 +300,7 @@ static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen
    unsigned int len;
    int ret;
    
-   while (NULL != fgets (line, buflen, fp))
+   while (NULL != fgets_with_counter (line, buflen, fp, &len))
      {
 	if (Base64_Table[(unsigned char) *line] == 0xFF)
 	  {
@@ -286,16 +308,6 @@ static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen
 	     else return -1;
 	  }
 	
-	len = strlen (line);
-	if (len)
-	  {
-	     if (line [len - 1] == '\n')
-	       {
-		  line[len - 1] = 0;
-		  len--;
-	       }
-	  }
-	
 	if ((len % 4) != 0)
 	  {
 	     if (decoding) return 0;
@@ -311,7 +323,7 @@ static int decode_base64 (FILE *fp, FILE *fpout, char *line, unsigned int buflen
 	
 	decoding = 1;
      }
-	
+   
    return 0;
 }
 
@@ -573,11 +585,11 @@ static int check_and_decode_base64 (FILE *fp, FILE *pipe_fp) /*{{{*/
    
    current_pmt = NULL;
    
-   while (NULL != fgets (line, sizeof (line), fp))
+   while (NULL != fgets_with_counter (line, sizeof (line), fp, NULL))
      {
 	try_again:
 	
-	if (*line == '\n')
+	if (*line == 0)
 	  {
 	     if (found_base64_signature)
 	       {
@@ -585,7 +597,9 @@ static int check_and_decode_base64 (FILE *fp, FILE *pipe_fp) /*{{{*/
 		  
 		  if (*Base64_Filename == 0)
 		    {
-		       sprintf (Base64_Filename, "unknown-64.%d", Base64_Unknown_Number);
+		       slrn_snprintf (Base64_Filename,
+				      sizeof (Base64_Filename),
+				      "unknown-64.%d", Base64_Unknown_Number);
 		       Base64_Unknown_Number++;
 		    }
 		  
@@ -726,6 +740,7 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
      {
 	char *file;
 	int mode = 0;
+	unsigned int buflen;
 
 	/* We are looking for 'begin ddd filename' */
 	if (strncmp (buf, "begin ", 6))
@@ -749,22 +764,22 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 	  return -1;
 
 	/* Now read parts of the file in. */
-	while (fgets (buf, sizeof(buf) - 1, fp) != NULL)
+	while (fgets_with_counter (buf, sizeof(buf) - 1, fp, &buflen) != NULL)
 	  {
 	     unsigned int len, write_len;
-	     unsigned int buflen;
 	     unsigned char out[60], *outp, *outmax, *b, *bmax;
 	     
-	     if (((*buf & 0x3F) == ' ') && (*(buf + 1) == '\n'))
+	     if (((*buf & 0x3F) == ' ') && ((*buf + 1) == '\0'))
 	       {
 		  /* we may be on the last line before the end.  Lets check */
-		  if (NULL == fgets (buf, sizeof(buf) - 1, fp))
+		  if (NULL == fgets_with_counter (buf, sizeof(buf) - 1, fp,
+						  &buflen))
 		    {
 		       slrn_error ("Unexpected end of file.\n");
 		       return -1;
 		    }
 		  
-		  if (!strcmp (buf, "end\n"))
+		  if (!strcmp (buf, "end"))
 		    {
 		       uudecode_fclose (outfp, pipe_fp);
 		       outfp = NULL;
@@ -778,9 +793,9 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 	     no_slop = 0;
 	     if ((len <= ' ')
 		 || (len >= (60 + ' '))
-		 || ((buflen = strlen (buf)) > 62) || (buflen < 2)
+		 || (buflen > 61) || (buflen < 1)
 		 || (no_slop
-		     && (((buflen - 2) % 4) != 0)))
+		     && (((buflen - 1) % 4) != 0)))
 	       {
 #if 0
 		  fprintf (stderr, "%s", buf);
@@ -798,7 +813,7 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 		  continue;
 	       }
 	     
-	     buflen -= 2;
+	     buflen--;
 	     len -= ' ';
 	     
 	     /* In general, I cannot make the test:
@@ -830,10 +845,6 @@ static int check_and_uudecode (FILE *fp, FILE *pipe_fp,
 		       continue;
 		    }
 	       }
-	     /* This may still be a bogus line, e.g., a header. */
-	     if (NULL != slrn_strchr (buf, ' '))
-	       continue;
-	     
 	     b = (unsigned char *) buf + 1;
 	     bmax = b + buflen;
 	     
@@ -962,7 +973,7 @@ int slrn_uudecode_file (char *file, char *newdir, /*{{{*/
 	if (newdir == NULL)
 	  newdir = "News";
 
-	slrn_make_home_dirname (newdir, newdir_buf);
+	slrn_make_home_dirname (newdir, newdir_buf, sizeof (newdir_buf));
 	newdir = newdir_buf;
      }
 #endif
