@@ -43,6 +43,7 @@
 #include "snprintf.h"
 
 char *Slrn_Version = SLRN_VERSION;
+int Slrn_Version_Number = SLRN_VERSION_NUMBER;
 char *Slrn_Date = SLRN_RELEASE_DATE;
 
 typedef struct 
@@ -76,7 +77,6 @@ static Compile_Option_Type Compile_Options [] =
      {"slang",		     	SLRN_HAS_SLANG},
      {"spoilers",    		SLRN_HAS_SPOILERS},
      {"strict_from",		SLRN_HAS_STRICT_FROM},
-     {"user_agent",  		SLRN_HAS_USER_AGENT},
      {"verbatim_marks",		SLRN_HAS_VERBATIM_MARKS},
      {NULL, 0}
 };
@@ -159,6 +159,9 @@ void slrn_show_version (void) /*{{{*/
 	fprintf (stdout, "\t* Note: This program was compiled against version %s.\n",
 		 make_slang_version (SLANG_VERSION));
      }
+#if defined(__DATE__) && defined(__TIME__)
+   fprintf (stdout, "Compiled at: %s %s\n", __DATE__, __TIME__);
+#endif
    fprintf (stdout, "Operating System: %s\n", os);
    
    show_compile_time_options ();
