@@ -43,11 +43,13 @@ Slrn_XOver_Type;
 extern char *slrn_get_extra_xover_header (char *);
 extern void slrn_map_xover_to_header (Slrn_XOver_Type *, Slrn_Header_Type *);
 
+#ifndef SLRNPULL_CODE
+extern int slrn_read_overview_fmt (void);
+
 extern void slrn_free_additional_headers (Slrn_Header_Line_Type *);
 extern void slrn_clear_requested_headers (void);
-extern void slrn_request_additional_header (char *);
+extern void slrn_request_additional_header (char *, int);
 
-#ifndef SLRNPULL_CODE
 extern int slrn_xover_for_msgid (char *, Slrn_XOver_Type *);
 extern int slrn_open_xover (int, int);
 extern int slrn_read_xover (Slrn_XOver_Type *);
